@@ -18,14 +18,13 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: bookmark.c,v 1.13 2003/12/28 16:02:01 masneyb Exp $";
+static const char cvsid[] = "$Id: bookmark.c,v 1.15 2004/09/14 10:16:52 masneyb Exp $";
 
 
 static int
 bookmark_parse_url (gftp_request * request, const char * url)
 {
   const char * pos;
-  int update_local;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (url != NULL, GFTP_EFATAL);
@@ -43,7 +42,7 @@ bookmark_parse_url (gftp_request * request, const char * url)
   else
     pos = url;
 
-  return (gftp_parse_bookmark (request, NULL, pos, &update_local));
+  return (gftp_parse_bookmark (request, NULL, pos, NULL));
 }
 
 
@@ -86,7 +85,8 @@ bookmark_init (gftp_request * request)
   request->parse_url = bookmark_parse_url;
   request->url_prefix = "bookmark";
   request->need_hostport = 0;
-  request->need_userpass = 0;
+  request->need_username = 0;
+  request->need_password = 0;
   request->use_cache = 0;
   request->always_connected = 0;
 
