@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: cache.c,v 1.18 2003/12/05 00:19:47 masneyb Exp $";
+static const char cvsid[] = "$Id: cache.c,v 1.21 2004/09/17 23:37:47 masneyb Exp $";
 
 struct gftp_cache_entry_tag
 {
@@ -127,7 +127,7 @@ gftp_new_cache_entry (gftp_request * request)
   time (&t);
   t += cache_ttl;
 
-  cachedir = expand_path (BASE_CONF_DIR "/cache");
+  cachedir = gftp_expand_path (NULL, BASE_CONF_DIR "/cache");
   if (access (cachedir, F_OK) == -1)
     {
       if (mkdir (cachedir, S_IRUSR | S_IWUSR | S_IXUSR) < 0)
@@ -205,7 +205,7 @@ gftp_find_cache_entry (gftp_request * request)
   gftp_generate_cache_description (request, description, sizeof (description),
                                    0);
 
-  indexfile = expand_path (BASE_CONF_DIR "/cache/index.db");
+  indexfile = gftp_expand_path (NULL, BASE_CONF_DIR "/cache/index.db");
   if ((indexfd = gftp_fd_open (NULL, indexfile, O_RDONLY, 0)) == -1)
     {
       g_free (indexfile);
@@ -269,7 +269,7 @@ gftp_clear_cache_files (void)
   gftp_cache_entry centry;
   int indexfd;
 
-  indexfile = expand_path (BASE_CONF_DIR "/cache/index.db");
+  indexfile = gftp_expand_path (NULL, BASE_CONF_DIR "/cache/index.db");
   if ((indexfd = gftp_fd_open (NULL, indexfile, O_RDONLY, 0)) == -1)
     {
       g_free (indexfile);
@@ -296,11 +296,11 @@ gftp_delete_cache_entry (gftp_request * request, char *descr,
                          int ignore_directory)
 {
   char *oldindexfile, *newindexfile, buf[BUFSIZ], description[BUFSIZ];
+  int indexfd, newfd, del_entry;
   gftp_getline_buffer * rbuf;
   gftp_cache_entry centry;
-  int indexfd, newfd;
+  size_t len;
   time_t now;
-  int remove;
  
   g_return_if_fail (request != NULL || descr != NULL);
 
@@ -318,14 +318,14 @@ gftp_delete_cache_entry (gftp_request * request, char *descr,
   else
     return;
 
-  oldindexfile = expand_path (BASE_CONF_DIR "/cache/index.db");
+  oldindexfile = gftp_expand_path (NULL, BASE_CONF_DIR "/cache/index.db");
   if ((indexfd = gftp_fd_open (NULL, oldindexfile, O_RDONLY, 0)) == -1)
     {
       g_free (oldindexfile);
       return;
     }
 
-  newindexfile = expand_path (BASE_CONF_DIR "/cache/index.db.new");
+  newindexfile = gftp_expand_path (NULL, BASE_CONF_DIR "/cache/index.db.new");
   if ((newfd = gftp_fd_open (request, newindexfile, O_WRONLY | O_CREAT, 
                              S_IRUSR | S_IWUSR)) == -1)
     {
@@ -340,22 +340,22 @@ gftp_delete_cache_entry (gftp_request * request, char *descr,
       if (gftp_parse_cache_line (request, &centry, buf) < 0)
         continue;
 
-      remove = 0;
+      del_entry = 0;
       if (centry.expiration_date < now)
-        remove = 1;
+        del_entry = 1;
       else if (ignore_directory)
         {
           if (strncmp (centry.url, description, strlen (description)) == 0)
-            remove = 1;
+            del_entry = 1;
         }
       else
         {
           if (strcmp (centry.url, description) == 0)
-            remove = 1;
+            del_entry = 1;
         }
 
  
-      if (remove)
+      if (del_entry)
         unlink (centry.file);
       else
         {
@@ -366,9 +366,10 @@ gftp_delete_cache_entry (gftp_request * request, char *descr,
 
           /* Make sure when we call gftp_get_line() that we pass the read size
              as sizeof(buf) - 1 so that we'll have room to put the newline */
-          buf[strlen (buf)] = '\n';
+          len = strlen (buf);
+          buf[len--] = '\n';
 
-          if (gftp_fd_write (NULL, buf, strlen (buf), newfd) < 0)
+          if (gftp_fd_write (NULL, buf, len, newfd) < 0)
             break;
         }
     }
