@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: config_file.c,v 1.38 2004/03/17 13:03:24 masneyb Exp $";
+static const char cvsid[] = "$Id: config_file.c,v 1.46 2004/11/03 10:51:32 masneyb Exp $";
 
 void
 gftp_add_bookmark (gftp_bookmarks_var * newentry)
@@ -79,29 +79,38 @@ gftp_add_bookmark (gftp_bookmarks_var * newentry)
       curpos++;
     }
 
-  /* Get the parent node */
-  if ((curpos = strrchr (newentry->path, '/')) == NULL)
-    preventry = gftp_bookmarks;
+  if (newentry->path[strlen (newentry->path) - 1] == '/')
+    {
+      newentry->path[strlen (newentry->path) - 1] = '\0';
+      newentry->isfolder = 1;
+    }
   else
     {
-      *curpos = '\0';
-      preventry = (gftp_bookmarks_var *)
-	g_hash_table_lookup (gftp_bookmarks_htable, newentry->path);
-      *curpos = '/';
-    }
+      /* Get the parent node */
+      if ((curpos = strrchr (newentry->path, '/')) == NULL)
+        preventry = gftp_bookmarks;
+      else
+        {
+          *curpos = '\0';
+          preventry = (gftp_bookmarks_var *)
+    	g_hash_table_lookup (gftp_bookmarks_htable, newentry->path);
+          *curpos = '/';
+        }
+    
+      if (preventry->children != NULL)
+        {
+          endentry = preventry->children;
+          while (endentry->next != NULL)
+    	endentry = endentry->next;
+          endentry->next = newentry;
+        }
+      else
+        preventry->children = newentry;
 
-  if (preventry->children != NULL)
-    {
-      endentry = preventry->children;
-      while (endentry->next != NULL)
-	endentry = endentry->next;
-      endentry->next = newentry;
+      newentry->prev = preventry;
+      newentry->next = NULL;
+      g_hash_table_insert (gftp_bookmarks_htable, newentry->path, newentry);
     }
-  else
-    preventry->children = newentry;
-  newentry->prev = preventry;
-  newentry->next = NULL;
-  g_hash_table_insert (gftp_bookmarks_htable, newentry->path, newentry);
 }
 
 
@@ -169,7 +178,7 @@ gftp_read_bookmarks (char *global_data_path)
   size_t len;
   int line;
 
-  if ((tempstr = expand_path (BOOKMARKS_FILE)) == NULL)
+  if ((tempstr = gftp_expand_path (NULL, BOOKMARKS_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad bookmarks file name %s\n"), BOOKMARKS_FILE);
       exit (1);
@@ -201,17 +210,17 @@ gftp_read_bookmarks (char *global_data_path)
   newentry = NULL;
   while (fgets (buf, sizeof (buf), bmfile))
     {
-     len = strlen (buf);
+      len = strlen (buf);
       if (len > 0 && buf[len - 1] == '\n')
-	buf[--len] = '\0';
+        buf[--len] = '\0';
       if (len > 0 && buf[len - 1] == '\r')
-	buf[--len] = '\0';
+        buf[--len] = '\0';
       line++;
 
       if (*buf == '[')
 	{
 	  newentry = g_malloc0 (sizeof (*newentry));
-	  for (; buf[len - 1] == ' ' || buf[len - 1] == ']'; buf[--len] = '\0');
+          for (; buf[len - 1] == ' ' || buf[len - 1] == ']'; buf[--len] = '\0');
 	  newentry->path = g_strdup (buf + 1);
 	  newentry->isfolder = 0;
 	  gftp_add_bookmark (newentry);
@@ -391,17 +400,14 @@ gftp_config_parse_args (char *str, int numargs, int lineno, char **first, ...)
       numargs--;
     }
   va_end (argp);
-  return (1);
+  return (ret);
 }
 
 
 static void *
 gftp_config_read_str (char *buf, int line)
 {
-  char *ret;
-
-  ret = g_strdup (buf);
-  return (ret);
+  return (g_strdup (buf));
 }
 
 
@@ -493,7 +499,7 @@ gftp_config_write_ext (FILE *fd, void *data)
 }
 
 
-gftp_config_list_vars gftp_config_list[] = {
+static gftp_config_list_vars gftp_config_list[] = {
   {"dont_use_proxy",	gftp_config_read_proxy,	gftp_config_write_proxy, 
    NULL, 0,
    N_("This section specifies which hosts are on the local subnet and won't need to go out the proxy server (if available). Syntax: dont_use_proxy=.domain or dont_use_proxy=network number/netmask")},
@@ -537,7 +543,7 @@ gftp_read_config_file (char *global_data_path)
   gftp_config_vars * tmpconfigvar;
   char **protocol_list;
   FILE *conffile;
-  int line, i;
+  int line, i, j;
   size_t len;
 
   gftp_global_options_htable = g_hash_table_new (string_hash_function, 
@@ -546,13 +552,14 @@ gftp_read_config_file (char *global_data_path)
   gftp_register_config_vars (gftp_global_config_vars);
 
   protocol_list = NULL;
-  for (i=0; gftp_protocols[i].register_options != NULL; i++)
+  for (i=0, j=0; gftp_protocols[i].register_options != NULL; i++)
     {
       if (gftp_protocols[i].shown)
         {
-          protocol_list = g_realloc (protocol_list, sizeof (char *) * (i + 2));
-          protocol_list[i] = gftp_protocols[i].name;
-          protocol_list[i + 1] = NULL;
+          protocol_list = g_realloc (protocol_list, sizeof (char *) * (j + 2));
+          protocol_list[j] = gftp_protocols[i].name;
+          protocol_list[j + 1] = NULL;
+          j++;
         }
 
       if (gftp_protocols[i].register_options != NULL)
@@ -575,7 +582,7 @@ gftp_read_config_file (char *global_data_path)
                            &gftp_config_list[i]);
     }
 
-  if ((tempstr = expand_path (CONFIG_FILE)) == NULL)
+  if ((tempstr = gftp_expand_path (NULL, CONFIG_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad config file name %s\n"), CONFIG_FILE);
       exit (1);
@@ -583,7 +590,7 @@ gftp_read_config_file (char *global_data_path)
 
   if (access (tempstr, F_OK) == -1)
     {
-      temp1str = expand_path (BASE_CONF_DIR);
+      temp1str = gftp_expand_path (NULL, BASE_CONF_DIR);
       if (access (temp1str, F_OK) == -1)
 	{
 	  if (mkdir (temp1str, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
@@ -659,7 +666,7 @@ gftp_read_config_file (char *global_data_path)
 	}
     }
 
-  if ((tempstr = expand_path (LOG_FILE)) == NULL)
+  if ((tempstr = gftp_expand_path (NULL, LOG_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad log file name %s\n"), LOG_FILE);
       exit (1);
@@ -725,7 +732,7 @@ gftp_write_bookmarks_file (void)
   bmhdr = N_("Bookmarks file for gFTP. Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>. Warning: Any comments that you add to this file WILL be overwritten");
   pwhdr = N_("Note: The passwords contained inside this file are scrambled. This algorithm is not secure. This is to avoid your password being easily remembered by someone standing over your shoulder while you're editing this file. Prior to this, all passwords were stored in plaintext.");
 
-  if ((tempstr = expand_path (BOOKMARKS_FILE)) == NULL)
+  if ((tempstr = gftp_expand_path (NULL, BOOKMARKS_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad bookmarks file name %s\n"), CONFIG_FILE);
       exit (1);
@@ -757,34 +764,40 @@ gftp_write_bookmarks_file (void)
       while (*tempstr == '/')
 	tempstr++;
 
-      if (tempentry->save_password && tempentry->pass != NULL)
-        password = gftp_scramble_password (tempentry->pass);
-      else
-        password = NULL;
-
-      fprintf (bmfile,
-	       "[%s]\nhostname=%s\nport=%d\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n",
-	       tempstr, tempentry->hostname == NULL ? "" : tempentry->hostname,
-	       tempentry->port, tempentry->protocol == NULL
-	       || *tempentry->protocol ==
-	       '\0' ? gftp_protocols[0].name : tempentry->protocol,
-	       tempentry->remote_dir == NULL ? "" : tempentry->remote_dir,
-	       tempentry->local_dir == NULL ? "" : tempentry->local_dir,
-	       tempentry->user == NULL ? "" : tempentry->user,
-	       password == NULL ? "" : password,
-	       tempentry->acct == NULL ? "" : tempentry->acct);
-
-      if (password != NULL)
-        g_free(password);
-
-      if (tempentry->local_options_vars != NULL)
+      if (tempentry->isfolder)
         {
-          for (i=0; i<tempentry->num_local_options_vars; i++)
-            {
-              gftp_option_types[tempentry->local_options_vars[i].otype].write_function (&tempentry->local_options_vars[i], buf, sizeof (buf), 1);
+          fprintf (bmfile, "[%s/]\n", tempstr);
+        }
+      else
+        {
+          if (tempentry->save_password && tempentry->pass != NULL)
+            password = gftp_scramble_password (tempentry->pass);
+          else
+            password = NULL;
 
-              fprintf (bmfile, "%s=%s\n", tempentry->local_options_vars[i].key,
-                       buf);
+          fprintf (bmfile,
+    	       "[%s]\nhostname=%s\nport=%d\nprotocol=%s\nremote directory=%s\nlocal directory=%s\nusername=%s\npassword=%s\naccount=%s\n",
+    	       tempstr, tempentry->hostname == NULL ? "" : tempentry->hostname,
+    	       tempentry->port, tempentry->protocol == NULL
+    	       || *tempentry->protocol ==
+    	       '\0' ? gftp_protocols[0].name : tempentry->protocol,
+    	       tempentry->remote_dir == NULL ? "" : tempentry->remote_dir,
+    	       tempentry->local_dir == NULL ? "" : tempentry->local_dir,
+    	       tempentry->user == NULL ? "" : tempentry->user,
+    	       password == NULL ? "" : password,
+    	       tempentry->acct == NULL ? "" : tempentry->acct);
+          if (password != NULL)
+            g_free(password);
+
+          if (tempentry->local_options_vars != NULL)
+            {
+              for (i=0; i<tempentry->num_local_options_vars; i++)
+                {
+                  gftp_option_types[tempentry->local_options_vars[i].otype].write_function (&tempentry->local_options_vars[i], buf, sizeof (buf), 1);
+    
+                  fprintf (bmfile, "%s=%s\n", tempentry->local_options_vars[i].key,
+                           buf);
+                }
             }
         }
 
@@ -814,7 +827,7 @@ gftp_write_config_file (void)
   FILE *conffile;
   int i;
 
-  if ((tempstr = expand_path (CONFIG_FILE)) == NULL)
+  if ((tempstr = gftp_expand_path (NULL, CONFIG_FILE)) == NULL)
     {
       printf (_("gFTP Error: Bad config file name %s\n"), CONFIG_FILE);
       exit (1);
@@ -1227,30 +1240,34 @@ gftp_config_file_read_textcombo (char *str, gftp_config_vars * cv, int line)
    gftp_option_type_enum in gftp.h */
 gftp_option_type_var gftp_option_types[] = {
   {gftp_config_file_read_text, gftp_config_file_write_text, 
-   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL},
+   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL,
+   NULL},
   {gftp_config_file_read_textcombo, gftp_config_file_write_text, 
-   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL},
+   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL,
+   NULL},
   {gftp_config_file_read_text, gftp_config_file_write_text, 
-   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL},
+   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL,
+   NULL},
   {gftp_config_file_read_text, gftp_config_file_write_hidetext, 
-   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL},
+   gftp_config_file_copy_text, gftp_config_file_compare_text, NULL, NULL, NULL,
+   NULL},
   {gftp_config_file_read_int, gftp_config_file_write_int, 
    gftp_config_file_copy_ptr_contents, gftp_config_file_compare_int, 
-   NULL, NULL, NULL},
+   NULL, NULL, NULL, NULL},
   {gftp_config_file_read_checkbox, gftp_config_file_write_int, 
    gftp_config_file_copy_ptr_contents, gftp_config_file_compare_int,
-   NULL, NULL, NULL},
+   NULL, NULL, NULL, NULL},
   {gftp_config_file_read_intcombo, gftp_config_file_write_intcombo, 
    gftp_config_file_copy_ptr_contents, gftp_config_file_compare_int, 
-   NULL, NULL, NULL},
+   NULL, NULL, NULL, NULL},
   {gftp_config_file_read_float, gftp_config_file_write_float, 
    gftp_config_file_copy_ptr_contents, gftp_config_file_compare_float,
-   NULL, NULL, NULL},
+   NULL, NULL, NULL, NULL},
   {gftp_config_file_read_color, gftp_config_file_write_color, 
    gftp_config_file_copy_color, gftp_config_file_compare_color, 
-   NULL, NULL, NULL},
-  {NULL, NULL, NULL, NULL, NULL, NULL},
-  {NULL, NULL, NULL, NULL, NULL, NULL}
+   NULL, NULL, NULL, NULL},
+  {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL},
+  {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}
 };
 
 
@@ -1323,10 +1340,10 @@ gftp_set_global_option (const char * key, const void *value)
       newconfigvar.value = nc_ptr;
       newconfigvar.flags &= ~GFTP_CVARS_FLAGS_DYNMEM;
 
-      ret = gftp_option_types[newconfigvar.otype].compare_function (&newconfigvar, tmpconfigvar);
+      ret = gftp_option_types[tmpconfigvar->otype].compare_function (&newconfigvar, tmpconfigvar);
       if (ret != 0)
         {
-          gftp_option_types[newconfigvar.otype].copy_function (&newconfigvar, tmpconfigvar);
+          gftp_option_types[tmpconfigvar->otype].copy_function (&newconfigvar, tmpconfigvar);
           gftp_configuration_changed = 1;
         }
     }
@@ -1351,7 +1368,7 @@ _gftp_set_option_value (gftp_config_vars * cv, const void * newval)
   newconfigvar.value = nc_ptr;
   newconfigvar.flags &= ~GFTP_CVARS_FLAGS_DYNMEM;
 
-  gftp_option_types[newconfigvar.otype].copy_function (&newconfigvar, cv);
+  gftp_option_types[cv->otype].copy_function (&newconfigvar, cv);
 }
 
 
@@ -1417,7 +1434,7 @@ gftp_set_bookmark_option (gftp_bookmarks_var * bm, const char * key,
       memcpy (&newconfigvar.value, &value, sizeof (newconfigvar.value));
       newconfigvar.flags &= ~GFTP_CVARS_FLAGS_DYNMEM;
 
-      ret = gftp_option_types[newconfigvar.otype].compare_function (&newconfigvar, tmpconfigvar);
+      ret = gftp_option_types[tmpconfigvar->otype].compare_function (&newconfigvar, tmpconfigvar);
       if (ret == 0)
         return;
       
@@ -1537,27 +1554,21 @@ gftp_bookmarks_destroy (gftp_bookmarks_var * bookmarks)
   tempentry = bookmarks;
   while (tempentry != NULL)
     {
-      gftp_free_bookmark (tempentry);
-
-      if (tempentry->path != NULL)
-        g_free (tempentry->path);
-
       if (tempentry->children != NULL)
+        tempentry = tempentry->children;
+      else
         {
-          tempentry = tempentry->children;
-          continue;
-        }
-
-      while (tempentry->next == NULL && tempentry->prev != NULL)
-        {
+          while (tempentry->next == NULL && tempentry->prev != NULL)
+            {
+              delentry = tempentry;
+              tempentry = tempentry->prev;
+              gftp_free_bookmark (delentry, 1);
+            }
+    
           delentry = tempentry;
-          tempentry = tempentry->prev;
-          g_free (delentry);
+          tempentry = tempentry->next;
+          gftp_free_bookmark (delentry, 1);
         }
-
-      delentry = tempentry;
-      tempentry = tempentry->next;
-      g_free (delentry);
     }
 }
 
