@@ -17,11 +17,15 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: gftp.h,v 1.88 2004/03/26 21:03:49 masneyb Exp $ */
+/* $Id: gftp.h,v 1.113 2005/01/16 15:48:24 masneyb Exp $ */
 
 #ifndef __GFTP_H
 #define __GFTP_H
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -70,6 +74,10 @@
 #include <stdint.h>
 #endif
 
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
 #ifdef HAVE_STRING_H
 #include <string.h>
 #else
@@ -99,7 +107,7 @@
 
 #ifdef HAVE_GETADDRINFO
 #define HAVE_IPV6
-#define GFTP_GET_AI_FAMILY(request)	(request != NULL && request->hostp != NULL ? request->hostp->ai_family : -1)
+#define GFTP_GET_AI_FAMILY(request)	((request) != NULL && (request)->hostp != NULL ? (request)->hostp->ai_family : -1)
 #else
 #define GFTP_GET_AI_FAMILY(request)	AF_INET
 #endif
@@ -153,6 +161,20 @@
 # define GFTP_LOG_FUNCTION_ATTRIBUTES
 #endif
 
+#if defined (_LARGEFILE_SOURCE) && !defined (__hppa__) && !defined (__hppa)
+#define GFTP_OFF_T_HEX_PRINTF_MOD	"%llx"
+#define GFTP_OFF_T_INTL_PRINTF_MOD	"%'lld"
+#define GFTP_OFF_T_PRINTF_MOD		"%lld"
+#define GFTP_OFF_T_11PRINTF_MOD		"%11lld"
+#define gftp_parse_file_size(str)	strtoll (str, NULL, 10)
+#else
+#define GFTP_OFF_T_HEX_PRINTF_MOD	"%lx"
+#define GFTP_OFF_T_INTL_PRINTF_MOD	"%'ld"
+#define GFTP_OFF_T_PRINTF_MOD		"%ld"
+#define GFTP_OFF_T_11PRINTF_MOD		"%11ld"
+#define gftp_parse_file_size(str)	strtol (str, NULL, 10)
+#endif
+
 /* Server types (used by FTP protocol from SYST command) */
 #define GFTP_DIRTYPE_UNIX	1
 #define GFTP_DIRTYPE_EPLF	2
@@ -205,6 +227,8 @@ typedef struct gftp_file_tag gftp_file;
 #define GFTP_SORT_COL_DATETIME			5
 #define GFTP_SORT_COL_ATTRIBS			6
 
+#define GFTP_IS_SPECIAL_DEVICE(mode)	(S_ISBLK (mode) || S_ISCHR (mode))
+
 struct gftp_file_tag 
 {
   char *file,			/* Our filename */
@@ -213,7 +237,6 @@ struct gftp_file_tag
 				   UTF-8 */
        *user,			/* User that owns it */
        *group,			/* Group that owns it */
-       *attribs,		/* Attribs (-rwxr-x-rx) */
        *destfile;		/* Full pathname to the destination for the 
                                    file transfer */
 
@@ -223,10 +246,8 @@ struct gftp_file_tag
   time_t datetime;		/* File date and time */
   off_t size,			/* Size of the file */
         startsize;		/* Size to start the transfer at */
-  unsigned int isdir : 1,	/* File type */
-               isexe : 1,
-               islink : 1,
-               selected : 1,	/* Is this file selected? */
+  mode_t st_mode;		/* File attributes */
+  unsigned int selected : 1,	/* Is this file selected? */
                was_sel : 1,	/* Was this file selected before  */
                shown : 1,	/* Is this file shown? */
                done_view : 1,	/* View the file when done transfering? */
@@ -333,6 +354,10 @@ typedef void (*gftp_logging_func)		( gftp_logging_level level,
 						  gftp_request * request, 
 						  const char *string, ... );
 
+#define GFTP_ANONYMOUS_USER			"anonymous"
+#define gftp_need_username(request)		((request)->need_username && ((request)->username == NULL || *(request)->username == '\0'))
+#define gftp_need_password(request)		((request)->need_password && (request)->username != NULL && *(request)->username != '\0' && strcasecmp ((request)->username, GFTP_ANONYMOUS_USER) != 0 && ((request)->password == NULL || *(request)->password == '\0'))
+
 struct gftp_request_tag 
 {
   int protonum;			/* Current number of the protocol this is 
@@ -357,11 +382,14 @@ struct gftp_request_tag
   /* One of these are used to lookup the IP address of the host we are
      connecting to */
 #if defined (HAVE_GETADDRINFO) && defined (HAVE_GAI_STRERROR)
-  struct addrinfo *hostp;
+  struct addrinfo *hostp,
+                  *current_hostp;
 #else
   struct hostent host, *hostp;
+  int curhost;
 #endif
 
+  int ai_family;
   int server_type;		/* The type of server we are connected to.
                                    See GFTP_DIRTYPE_* above */
   unsigned int free_hostp : 1, 		/* Should we free the hostp structure 
@@ -369,7 +397,8 @@ struct gftp_request_tag
                use_proxy : 1,
                always_connected : 1,
                need_hostport : 1,
-               need_userpass : 1,
+               need_username : 1,
+               need_password : 1,
                use_cache : 1,           /* Enable or disable the cache */
                cached : 1,              /* Is this directory listing cached? */
                cancel : 1,		/* If a signal is received, should
@@ -421,10 +450,17 @@ struct gftp_request_tag
 					  size_t size );
   int (*end_transfer) 			( gftp_request * request );
   int (*abort_transfer) 		( gftp_request * request );
+  int (*stat_filename) 			( gftp_request * request,
+					  const char *filename,
+					  mode_t * mode );
   int (*list_files) 			( gftp_request * request );
   int (*get_next_file)			( gftp_request * request, 
 					  gftp_file *fle, 
 					  int fd );
+  ssize_t (*get_next_dirlist_line)	( gftp_request * request, 
+					  int fd,
+					  char *buf,
+					  size_t buflen );
   off_t (*get_file_size) 		( gftp_request * request, 
 					  const char *filename );
   int (*chdir)				( gftp_request * request, 
@@ -440,11 +476,12 @@ struct gftp_request_tag
 					  const char *newname );
   int (*chmod)				( gftp_request * request, 
 					  const char *filename, 
-					  int mode );
+					  mode_t mode );
   int (*set_file_time)			( gftp_request * request, 
 					  const char *filename, 
 					  time_t datettime );
   int (*site)				( gftp_request * request, 
+					  int specify_site,
 					  const char *filename );
   int (*parse_url)			( gftp_request * request,
 					  const char *url );
@@ -529,6 +566,7 @@ typedef struct supported_gftp_protocols_tag
   int (*init) (gftp_request * request);		/* Init function */
   void (*register_options) (void);		/* Protocol options */
   char *url_prefix;				/* URL Prefix */
+  unsigned int default_port;			/* Default port */
   unsigned int shown : 1,			/* Whether this protocol is 
                                                    shown or not to the user in 
                                                    the protocol dropdown box */
@@ -542,8 +580,9 @@ typedef struct gftp_bookmarks_tag gftp_bookmarks_var;
 
 struct gftp_bookmarks_tag 
 {
-  char *path;                  /* Path */
-  char *hostname,              /* Our actual internet hostname */
+  char *path,                  /* Path */
+       *oldpath,               /* Old path name */
+       *hostname,              /* Our actual internet hostname */
        *protocol,              /* Protocol we will connect through */
        *remote_dir,            /* Initial directory */
        *local_dir,             /* Init local directory */
@@ -572,7 +611,7 @@ typedef struct gftp_file_extensions_tag
         *filename,              /* The xpm file to display */
         *view_program,          /* The program used to view this file */
         *ascii_binary;          /* Is this a ASCII transfer or a BINARY transfer */
-   int stlen;                   /* How long is the file extension. */
+   unsigned int stlen;          /* How long is the file extension. */
 } gftp_file_extensions;
 
 
@@ -692,7 +731,8 @@ char *insert_commas 			( off_t number,
 
 char *alltrim 				( char *str );
 
-char *expand_path 			( const char *src );
+char *gftp_expand_path 			( gftp_request * request,
+					  const char *src );
 
 void make_nonnull 			( char **str );
 
@@ -711,8 +751,6 @@ guint string_hash_function		( gconstpointer key );
 
 void free_file_list			( GList * filelist );
 
-void free_fdata				( gftp_file * fle );
-
 gftp_file * copy_fdata 			( gftp_file * fle );
 
 int compare_request 			( gftp_request * request1, 
@@ -729,15 +767,14 @@ GList * gftp_sort_filelist 		( GList * filelist,
 					  int column, 
 					  int asds );
 
-mode_t gftp_parse_attribs 		( char *attribs );
-
 char * gftp_gen_ls_string 		( gftp_file * fle, 
 					  char *file_prefixstr, 
 					  char *file_suffixstr );
 
 char * base64_encode 			( char *str );
 
-void gftp_free_bookmark 		( gftp_bookmarks_var * entry );
+void gftp_free_bookmark 		( gftp_bookmarks_var * entry,
+					  int free_node );
 
 void gftp_shutdown			( void );
 
@@ -745,11 +782,10 @@ GList * get_next_selection 		( GList * selection,
 					  GList ** list, 
 					  int *curnum );
 
-char * gftp_build_path 			( const char *first_element,
+char * gftp_build_path 			( gftp_request * request,
+					  const char *first_element,
 					  ... );
 
-off_t gftp_parse_file_size 		( char *str );
-
 void gftp_locale_init 			( void );
 
 char * gftp_scramble_password		( const char *password );
@@ -759,6 +795,8 @@ char * gftp_descramble_password		( const char *password );
 int gftp_get_transfer_action 		( gftp_request * request,
 					  gftp_file * fle );
 
+char * gftp_get_share_dir 		( void );
+
 /* mkstemps.c */
 int mkstemps 				( char *template,
 					  int suffix_len );
@@ -771,6 +809,7 @@ int mkstemps 				( char *template,
 #define GFTP_LOCAL_NUM				4
 #define GFTP_SSHV2_NUM				5
 #define GFTP_BOOKMARK_NUM			6
+#define GFTP_FSP_NUM				7
 
 #define GFTP_IS_CONNECTED(request)		((request) != NULL && \
                                                  ((request)->datafd > 0 || \
@@ -814,6 +853,10 @@ int bookmark_init 			( gftp_request * request );
 
 void bookmark_register_module		( void );
 
+int fsp_init				( gftp_request * request );
+
+void fsp_register_module		( void );
+
 gftp_request *gftp_request_new 		( void );
 
 void gftp_request_destroy 		( gftp_request * request,
@@ -822,7 +865,8 @@ void gftp_request_destroy 		( gftp_request * request,
 void gftp_copy_param_options		( gftp_request * dest_request,
 					  gftp_request * src_request );
 
-void gftp_file_destroy 			( gftp_file *file );
+void gftp_file_destroy 			( gftp_file *file,
+					  int free_it );
 
 int gftp_connect 			( gftp_request * request );
 
@@ -880,6 +924,10 @@ int gftp_end_transfer 			( gftp_request * request );
 
 int gftp_abort_transfer 		( gftp_request * request );
 
+int gftp_stat_filename			( gftp_request * request,
+					  const char *filename,
+					  mode_t * mode );
+
 void gftp_set_hostname 			( gftp_request * request, 
 					  const char *hostname );
 
@@ -913,13 +961,14 @@ int gftp_rename_file 			( gftp_request * request,
 
 int gftp_chmod 				( gftp_request * request, 
 					  const char *file, 
-					  int mode );
+					  mode_t mode );
 
 int gftp_set_file_time 			( gftp_request * request, 
 					  const char *file, 
 					  time_t datetime );
 
 char gftp_site_cmd 			( gftp_request * request, 
+					  int specify_site,
 					  const char *command );
 
 off_t gftp_get_file_size 		( gftp_request * request, 
@@ -933,7 +982,8 @@ time_t parse_time 			( char *str,
 
 int gftp_parse_ls 			( gftp_request * request,
 					  const char *lsoutput, 
-					  gftp_file *fle );
+					  gftp_file *fle,
+					  int fd );
 
 int gftp_get_all_subdirs 		( gftp_transfer * transfer, 
 					  void (*update_func) 
@@ -942,7 +992,7 @@ int gftp_get_all_subdirs 		( gftp_transfer * transfer,
 int gftp_connect_server 		( gftp_request * request, 
 					  char *service,
 					  char *proxy_hostname,
-					  int proxy_port );
+					  unsigned int proxy_port );
 
 #if !defined (HAVE_GETADDRINFO) || !defined (HAVE_GAI_STRERROR)
 
@@ -1004,16 +1054,20 @@ int gftp_fd_open 			( gftp_request * request,
 
 void gftp_setup_startup_directory 	( gftp_request * request );
 
+unsigned int gftp_protocol_default_port	( gftp_request * request );
+
 /* pty.c */
 char * gftp_get_pty_impl 		( void );
 
-pid_t gftp_exec_with_new_pty 		( gftp_request * request, 
-					  int *fdm, 
+pid_t gftp_exec 			( gftp_request * request,
+					  int *fdm,
+					  int *ptymfd,
 					  char **args );
 
-pid_t gftp_exec_without_new_pty 	( gftp_request * request, 
-					  int *fdm, 
-					  char **args );
+char *gftp_convert_attributes_from_mode_t ( mode_t mode );
+
+mode_t gftp_convert_attributes_to_mode_t ( char *attribs );
+
 
 #ifdef USE_SSL
 /* sslcommon.c */
@@ -1034,5 +1088,17 @@ ssize_t gftp_ssl_write 			( gftp_request * request,
 					  int fd );
 #endif /* USE_SSL */
 
+/* UI dependent functions that must be implemented */
+int gftpui_protocol_ask_yes_no 		( gftp_request * request,
+					  char *title,
+					  char *question );
+
+char *gftpui_protocol_ask_user_input	( gftp_request * request,
+					  char *title,
+					  char *question,
+					  int shown );
+
+void gftpui_protocol_update_timeout 	( gftp_request * request );
+
 #endif
 
