@@ -17,14 +17,14 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: httpcommon.h,v 1.4 2003/11/03 02:11:17 masneyb Exp $ */
+/* $Id: httpcommon.h,v 1.5 2004/07/27 02:46:54 masneyb Exp $ */
 
 typedef struct rfc2068_params_tag
 {
   gftp_getline_buffer * rbuf;
-  unsigned long read_bytes,
-                content_length;
-  long chunk_size;
+  unsigned long read_bytes;
+  off_t chunk_size,
+        content_length;
   unsigned int chunked_transfer : 1,
                eof : 1;
   ssize_t (*real_read_function) ( gftp_request * request,
