@@ -20,14 +20,15 @@
 #include "gftp.h"
 #include "httpcommon.h"
 
-static const char cvsid[] = "$Id: https.c,v 1.6 2004/03/02 02:29:10 masneyb Exp $";
+static const char cvsid[] = "$Id: https.c,v 1.7 2004/07/27 02:46:54 masneyb Exp $";
 
 #ifdef USE_SSL
 static int
 https_get_next_file (gftp_request * request, gftp_file * fle, int fd)
 {
   rfc2068_params * params;
-  int ret, resetptr;
+  int resetptr;
+  size_t ret;
 
   params = request->protocol_data;
   if (request->cached)
