@@ -18,7 +18,7 @@
 /*****************************************************************************/
 
 #include "gftp.h"
-static const char cvsid[] = "$Id: local.c,v 1.31 2004/03/18 12:01:57 masneyb Exp $";
+static const char cvsid[] = "$Id: local.c,v 1.39 2004/09/14 10:16:52 masneyb Exp $";
 
 typedef struct local_protocol_data_tag
 {
@@ -59,8 +59,12 @@ local_connect (gftp_request * request)
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
 
-  if (request->directory)
+  if (request->directory != NULL)
     {
+      if (getcwd (tempstr, sizeof (tempstr)) != NULL &&
+          strcmp (tempstr, request->directory) == 0)
+        return (0);
+
       if (chdir (request->directory) != 0)
         {
           request->logging_function (gftp_logging_error, request,
@@ -220,71 +224,17 @@ local_end_transfer (gftp_request * request)
 }
 
 
-static char *
-make_text_mode (mode_t mode)
+static int
+local_stat_filename (gftp_request * request, const char *filename,
+                     mode_t * mode)
 {
-  char *str;
+  struct stat st;
 
-  str = g_malloc0 (11);
-  
-  str[0] = '?';
-  if (S_ISREG (mode))
-    str[0] = '-';
+  if (stat (filename, &st) != 0)
+    return (GFTP_ERETRYABLE);
 
-  if (S_ISLNK (mode))
-    str[0] = 'l';
-
-  if (S_ISBLK (mode))
-    str[0] = 'b';
-
-  if (S_ISCHR (mode))
-    str[0] = 'c';
-
-  if (S_ISFIFO (mode))
-    str[0] = 'p';
-
-  if (S_ISSOCK (mode))
-    str[0] = 's';
-
-  if (S_ISDIR (mode))
-    str[0] = 'd';
-
-  str[1] = mode & S_IRUSR ? 'r' : '-';
-  str[2] = mode & S_IWUSR ? 'w' : '-';
-
-  if ((mode & S_ISUID) && (mode & S_IXUSR))
-    str[3] = 's';
-  else if (mode & S_ISUID)
-    str[3] = 'S';
-  else if (mode & S_IXUSR)
-    str[3] = 'x';
-  else
-    str[3] = '-';
-    
-  str[4] = mode & S_IRGRP ? 'r' : '-';
-  str[5] = mode & S_IWGRP ? 'w' : '-';
-
-  if ((mode & S_ISGID) && (mode & S_IXGRP))
-    str[6] = 's';
-  else if (mode & S_ISGID)
-    str[6] = 'S';
-  else if (mode & S_IXGRP)
-    str[6] = 'x';
-  else
-    str[6] = '-';
-
-  str[7] = mode & S_IROTH ? 'r' : '-';
-  str[8] = mode & S_IWOTH ? 'w' : '-';
-
-  if ((mode & S_ISVTX) && (mode & S_IXOTH))
-    str[9] = 't';
-  else if (mode & S_ISVTX)
-    str[9] = 'T';
-  else if (mode & S_IXOTH)
-    str[9] = 'x';
-  else
-    str[9] = '-';
-  return (str);
+  *mode = st.st_mode;
+  return (0);
 }
 
 
@@ -353,21 +303,14 @@ local_get_next_file (gftp_request * request, gftp_file * fle, int fd)
       g_hash_table_insert (lpd->grouphash, GUINT_TO_POINTER (st.st_gid), group);
     }
 
-  fle->attribs = make_text_mode (fst.st_mode);
+  fle->st_mode = fst.st_mode;
   fle->datetime = st.st_mtime;
 
-  if ((fle->attribs[0] == 'b' || fle->attribs[0] == 'u' ||
-       fle->attribs[0] == 'c'))
+  if (GFTP_IS_SPECIAL_DEVICE (fle->st_mode))
     fle->size = (off_t) st.st_rdev;
   else
     fle->size = fst.st_size;
 
-  fle->isdir = S_ISDIR (fst.st_mode);
-  fle->islink = S_ISLNK (st.st_mode);
-  fle->isexe = (fst.st_mode & S_IXUSR) || 
-               (fst.st_mode & S_IXGRP) ||
-               (fst.st_mode & S_IXOTH);
-
   return (1);
 }
 
@@ -565,29 +508,23 @@ local_rename (gftp_request * request, const char *oldname,
 
 
 static int
-local_chmod (gftp_request * request, const char *file, int mode)
+local_chmod (gftp_request * request, const char *file, mode_t mode)
 {
-  char buf[10];
-  int newmode;
-
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
 
-  g_snprintf (buf, sizeof (buf), "%d", mode);
-  newmode = strtol (buf, NULL, 8);
-
-  if (chmod (file, newmode) == 0) 
+  if (chmod (file, mode) == 0) 
     {
       request->logging_function (gftp_logging_misc, request, 
-                                 _("Successfully changed mode of %s to %d\n"),
+                                 _("Successfully changed mode of %s to %o\n"),
                                  file, mode);
       return (0);
     }
   else 
     {
       request->logging_function (gftp_logging_error, request, 
-                          _("Error: Could not change mode of %s to %d: %s\n"),
+                          _("Error: Could not change mode of %s to %o: %s\n"),
                           file, mode, g_strerror (errno));
       return (GFTP_ERETRYABLE);
     }
@@ -604,7 +541,8 @@ local_set_file_time (gftp_request * request, const char *file,
   g_return_val_if_fail (request->protonum == GFTP_LOCAL_NUM, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
 
-  time_buf.modtime = time_buf.actime = datetime;
+  time_buf.modtime = datetime;
+  time_buf.actime = datetime;
   return (utime (file, &time_buf) == 0 ? 0 : GFTP_ERETRYABLE);
 }
 
@@ -652,8 +590,10 @@ local_init (gftp_request * request)
   request->put_next_file_chunk = NULL;
   request->end_transfer = local_end_transfer;
   request->abort_transfer = local_end_transfer; /* NOTE: uses end_transfer */
+  request->stat_filename = local_stat_filename;
   request->list_files = local_list_files;
   request->get_next_file = local_get_next_file;
+  request->get_next_dirlist_line = NULL;
   request->get_file_size = local_get_file_size;
   request->chdir = local_chdir;
   request->rmdir = local_rmdir;
@@ -668,7 +608,8 @@ local_init (gftp_request * request)
   request->swap_socks = NULL;
   request->url_prefix = "file";
   request->need_hostport = 0;
-  request->need_userpass = 0;
+  request->need_username = 0;
+  request->need_password = 0;
   request->use_cache = 0;
   request->always_connected = 1;
 
