@@ -20,7 +20,7 @@
 #include "gftp.h"
 #include "httpcommon.h"
 
-static const char cvsid[] = "$Id: rfc2068.c,v 1.29 2003/08/03 18:44:29 masneyb Exp $";
+static const char cvsid[] = "$Id: rfc2068.c,v 1.53 2004/10/29 20:05:32 masneyb Exp $";
 
 static gftp_config_vars config_vars[] =
 {
@@ -53,15 +53,12 @@ static int
 rfc2068_connect (gftp_request * request)
 {
   char *proxy_hostname, *proxy_config;
-  rfc2068_params * params;
-  int proxy_port, ret;
+  intptr_t proxy_port;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
   g_return_val_if_fail (request->hostname != NULL, GFTP_EFATAL);
 
-  params = request->protocol_data;
-
   if (request->datafd > 0)
     return (0);
 
@@ -147,7 +144,7 @@ rfc2068_read_response (gftp_request * request)
                                 sizeof (tempstr), request->datafd)) < 0)
         return (ret);
 
-      if (sscanf ((char *) tempstr, "%lx", &params->chunk_size) != 1)
+      if (sscanf (tempstr, GFTP_OFF_T_HEX_PRINTF_MOD, &params->chunk_size) != 1)
         {
           request->logging_function (gftp_logging_recv, request,
                                      _("Received wrong response from server, disconnecting\nInvalid chunk size '%s' returned by the remote server\n"), 
@@ -181,11 +178,11 @@ static off_t
 rfc2068_send_command (gftp_request * request, const void *command, size_t len)
 {
   char *tempstr, *str, *proxy_hostname, *proxy_username, *proxy_password;
-  int proxy_port, conn_ret;
+  intptr_t proxy_port;
+  int conn_ret;
   ssize_t ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
   g_return_val_if_fail (command != NULL, GFTP_EFATAL);
 
   if (request->datafd < 0 && (conn_ret = rfc2068_connect (request)) != 0)
@@ -250,7 +247,6 @@ static void
 rfc2068_disconnect (gftp_request * request)
 {
   g_return_if_fail (request != NULL);
-  g_return_if_fail (request->protonum == GFTP_HTTP_NUM);
 
   if (request->datafd > 0)
     {
@@ -272,13 +268,14 @@ static off_t
 rfc2068_get_file (gftp_request * request, const char *filename, int fd,
                   off_t startsize)
 {
-  char *tempstr, *oldstr, *pos;
-  int restarted, use_http11;
+  char *tempstr, *oldstr, *hf;
   rfc2068_params * params;
+  intptr_t use_http11;
+  int restarted;
+  size_t len;
   off_t size;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
   g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
 
   params = request->protocol_data;
@@ -288,39 +285,28 @@ rfc2068_get_file (gftp_request * request, const char *filename, int fd,
   if (fd > 0)
     request->datafd = fd;
 
+  hf = gftp_build_path (request, request->hostname, filename, NULL);
+
   if (request->username == NULL || *request->username == '\0')
-    tempstr = g_strconcat ("GET ", request->url_prefix, "://", 
-                     request->hostname, "/", filename, 
+    tempstr = g_strconcat ("GET ", request->url_prefix, "://", hf,
                      use_http11 ? " HTTP/1.1\n" : " HTTP/1.0\n", NULL);
   else 
     tempstr = g_strconcat ("GET ", request->url_prefix, "://", 
-                     request->username, "@", request->hostname, "/", filename, 
+                     request->username, "@", hf,
                      use_http11 ? " HTTP/1.1\n" : " HTTP/1.0\n", NULL);
 
-  if ((pos = strstr (tempstr, "://")) != NULL)
-    remove_double_slashes (pos + 3);
-  else
-    remove_double_slashes (tempstr);
+  g_free (hf);
 
   if (use_http11 && startsize > 0)
     {
-#if defined (_LARGEFILE_SOURCE)
       request->logging_function (gftp_logging_misc, request,
-                              _("Starting the file transfer at offset %lld\n"),
+                              _("Starting the file transfer at offset " GFTP_OFF_T_PRINTF_MOD "\n"),
                               startsize);
 
       oldstr = tempstr;
-      tempstr = g_strdup_printf ("%sRange: bytes=%lld-\n", tempstr, startsize);
+      tempstr = g_strdup_printf ("%sRange: bytes=" GFTP_OFF_T_PRINTF_MOD "-\n",
+                                 tempstr, startsize);
       g_free (oldstr);
-#else
-      request->logging_function (gftp_logging_misc, request,
-			       _("Starting the file transfer at offset %ld\n"), 
-                               startsize);
-
-      oldstr = tempstr;
-      tempstr = g_strdup_printf ("%sRange: bytes=%ld-\n", tempstr, startsize);
-      g_free (oldstr);
-#endif
     }
 
   size = rfc2068_send_command (request, tempstr, strlen (tempstr));
@@ -329,11 +315,10 @@ rfc2068_get_file (gftp_request * request, const char *filename, int fd,
     return (size);
 
   restarted = 0;
-  if (strlen (request->last_ftp_response) > 9 
-      && strncmp (request->last_ftp_response + 9, "206", 3) == 0)
+  len = strlen (request->last_ftp_response);
+  if (len  > 9 && strncmp (request->last_ftp_response + 9, "206", 3) == 0)
     restarted = 1;
-  else if (strlen (request->last_ftp_response) < 9 ||
-           strncmp (request->last_ftp_response + 9, "200", 3) != 0)
+  else if (len < 9 || strncmp (request->last_ftp_response + 9, "200", 3) != 0)
     {
       request->logging_function (gftp_logging_error, request,
 			         _("Cannot retrieve file %s\n"), filename);
@@ -353,14 +338,25 @@ rfc2068_get_next_file_chunk (gftp_request * request, char *buf, size_t size)
   size_t len;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
 
   params = request->protocol_data;
+  if (params->rbuf != NULL && params->rbuf->curpos != NULL)
+    {
+      len = params->rbuf->cur_bufsize > size ? size : params->rbuf->cur_bufsize;
+      memcpy (buf, params->rbuf->curpos, len);
 
-  if ((len = request->read_function (request, buf, size, request->datafd)) < 0)
-    return ((ssize_t) len);
+      if (len == params->rbuf->cur_bufsize)
+        gftp_free_getline_buffer (&params->rbuf);
+      else
+        {
+          params->rbuf->curpos += len;
+          params->rbuf->cur_bufsize -= len;
+        }
 
-  return (len);
+      return (len);
+    }
+
+  return (request->read_function (request, buf, size, request->datafd));
 }
 
 
@@ -370,7 +366,6 @@ rfc2068_end_transfer (gftp_request * request)
   rfc2068_params * params;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
 
   if (request->datafd < 0)
     return (GFTP_EFATAL);
@@ -391,30 +386,29 @@ static int
 rfc2068_list_files (gftp_request * request)
 {
   rfc2068_params *params;
-  char *tempstr, *pos;
-  int use_http11;
+  char *tempstr, *hd;
+  intptr_t use_http11;
   off_t ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
 
   params = request->protocol_data;
   gftp_lookup_request_option (request, "use_http11", &use_http11);
 
+  if (strncmp (request->directory, "/", strlen (request->directory)) == 0)
+    hd = g_strdup (request->hostname);
+  else
+    hd = gftp_build_path (request, request->hostname, request->directory, NULL);
+
   if (request->username == NULL || *request->username == '\0')
-    tempstr = g_strconcat ("GET ", request->url_prefix, "://", 
-                           request->hostname, "/", request->directory, 
+    tempstr = g_strconcat ("GET ", request->url_prefix, "://", hd,
                            use_http11 ? "/ HTTP/1.1\n" : "/ HTTP/1.0\n", NULL);
   else
     tempstr = g_strconcat ("GET ", request->url_prefix, "://", 
-                           request->username, "@", request->hostname, "/", 
-                           request->directory, 
+                           request->username, "@", hd,
                            use_http11 ? "/ HTTP/1.1\n" : "/ HTTP/1.0\n", NULL);
 
-  if ((pos = strstr (tempstr, "://")) != NULL)
-    remove_double_slashes (pos + 3);
-  else
-    remove_double_slashes (tempstr);
+  g_free (hd);
 
   ret = rfc2068_send_command (request, tempstr, strlen (tempstr));
   g_free (tempstr);
@@ -439,32 +433,26 @@ rfc2068_list_files (gftp_request * request)
 static off_t 
 rfc2068_get_file_size (gftp_request * request, const char *filename)
 {
-  rfc2068_params *params;
-  char *tempstr, *pos;
-  int use_http11;
+  char *tempstr, *hf;
+  intptr_t use_http11;
   off_t size;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
   g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
 
-  params = request->protocol_data;
   gftp_lookup_request_option (request, "use_http11", &use_http11);
 
+  hf = gftp_build_path (request, request->hostname, filename, NULL);
+
   if (request->username == NULL || *request->username == '\0')
-    tempstr = g_strconcat ("HEAD ", request->url_prefix, "://", 
-                           request->hostname, "/", filename, 
+    tempstr = g_strconcat ("HEAD ", request->url_prefix, "://", hf,
                            use_http11 ? " HTTP/1.1\n" : " HTTP/1.0\n", NULL);
   else
     tempstr = g_strconcat ("HEAD ", request->url_prefix, "://", 
-                           request->username, "@", request->hostname, "/", 
-                           filename, 
+                           request->username, "@", hf,
                            use_http11 ? " HTTP/1.1\n" : " HTTP/1.0\n", NULL);
 
-  if ((pos = strstr (tempstr, "://")) != NULL)
-    remove_double_slashes (pos + 3);
-  else
-    remove_double_slashes (tempstr);
+  g_free (hf);
 
   size = rfc2068_send_command (request, tempstr, strlen (tempstr));
   g_free (tempstr);
@@ -498,13 +486,13 @@ parse_html_line (char *tempstr, gftp_file * fle)
 
   /* Copy file attributes. Just about the only thing we can get is whether it
      is a directory or not */
-  fle->attribs = g_strdup ("----------");
   if (*(pos - 1) == '/')
     {
+      fle->st_mode = S_IFDIR | S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
       *(pos - 1) = '\0';
-      *fle->attribs = 'd';
-      fle->isdir = 1;
     }
+  else
+    fle->st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
 
   /* Copy filename */
   if (strchr (stpos, '/') != NULL || strncmp (stpos, "mailto:", 7) == 0 ||
@@ -567,11 +555,8 @@ parse_html_line (char *tempstr, gftp_file * fle)
 
   kpos = strchr (pos, 'k');
   mpos = strchr (pos, 'M');
-  if (kpos == NULL)
-    stpos = mpos;
-  else if (mpos == NULL)
-    stpos = kpos;
-  else if (kpos < stpos)
+
+  if (kpos != NULL && (mpos == NULL || (kpos < mpos)))
     stpos = kpos;
   else
     stpos = mpos;
@@ -607,11 +592,10 @@ rfc2068_get_next_file (gftp_request * request, gftp_file * fle, int fd)
 {
   rfc2068_params * params;
   char tempstr[8192];
-  ssize_t len;
+  size_t len;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
   g_return_val_if_fail (fle != NULL, GFTP_EFATAL);
 
   params = request->protocol_data;
@@ -630,14 +614,14 @@ rfc2068_get_next_file (gftp_request * request, gftp_file * fle, int fd)
         return (ret);
 
       if (parse_html_line (tempstr, fle) == 0 || fle->file == NULL)
-	gftp_file_destroy (fle);
+	gftp_file_destroy (fle, 0);
       else
 	break;
     }
 
   if (fle->file == NULL)
     {
-      gftp_file_destroy (fle);
+      gftp_file_destroy (fle, 0);
       return (0);
     }
 
@@ -658,7 +642,6 @@ rfc2068_chdir (gftp_request * request, const char *directory)
   char *tempstr, *olddir;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
-  g_return_val_if_fail (request->protonum == GFTP_HTTP_NUM, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
 
   if (request->directory != directory)
@@ -668,11 +651,11 @@ rfc2068_chdir (gftp_request * request, const char *directory)
       if (*directory != '/' && request->directory != NULL)
         {
           tempstr = g_strconcat (request->directory, "/", directory, NULL);
-          request->directory = expand_path (tempstr);
+          request->directory = gftp_expand_path (request, tempstr);
           g_free (tempstr);
         }
       else
-        request->directory = expand_path (directory);
+        request->directory = gftp_expand_path (request, directory);
 
       if (olddir != NULL)
         g_free (olddir);
@@ -716,13 +699,15 @@ rfc2068_destroy (gftp_request * request)
 static ssize_t 
 rfc2068_chunked_read (gftp_request * request, void *ptr, size_t size, int fd)
 {
-  size_t read_size, begin_ptr_len, current_size;
-  char *stpos, *crlfpos;
+  size_t read_size, begin_ptr_len, current_size, crlfsize;
   rfc2068_params * params;
+  char *stpos, *crlfpos;
   void *read_ptr_pos;
   ssize_t retval;
+  size_t sret;
 
   params = request->protocol_data;
+  params->read_ref_cnt++;
 
   if (params->extra_read_buffer != NULL)
     {
@@ -752,7 +737,10 @@ rfc2068_chunked_read (gftp_request * request, void *ptr, size_t size, int fd)
       if (params->content_length > 0)
         {
           if (params->content_length == params->read_bytes)
-            return (0);
+            {
+              params->read_ref_cnt--;
+              return (0);
+            }
 
           if (read_size + params->read_bytes > params->content_length)
             read_size = params->content_length - params->read_bytes;
@@ -764,36 +752,53 @@ rfc2068_chunked_read (gftp_request * request, void *ptr, size_t size, int fd)
 
   if (read_size > 0 && !params->eof)
     {
-      read_size--; /* decrement by one so that we can put the NUL character in
-                      the buffer */
-      retval = params->real_read_function (request, read_ptr_pos, read_size, fd);
+      if (size == read_size)
+        read_size--; /* decrement by one so that we can put the NUL character
+                        in the buffer */
 
-      if (retval > 0)
-        params->read_bytes += retval;
-      else if (retval == 0)
-        return (retval);
+      retval = params->real_read_function (request, read_ptr_pos, read_size,
+                                           fd);
 
-      if (params->chunk_size > 0 && retval > 0)
+      if (retval <= 0)
         {
-          params->chunk_size -= retval;
+          params->read_ref_cnt--;
           return (retval);
         }
 
-      retval += begin_ptr_len;
+      params->read_bytes += retval;
+      if (params->chunk_size > 0)
+        {
+          params->chunk_size -= retval;
+          params->read_ref_cnt--;
+          return (retval);
+        }
+
+      sret = retval + begin_ptr_len;
     }
   else
-    retval = begin_ptr_len;
+    sret = begin_ptr_len;
 
-  ((char *) ptr)[retval] = '\0';
+  ((char *) ptr)[sret] = '\0';
 
-  if (!params->chunked_transfer || retval <= 0)
-    return (retval);
+  if (!params->chunked_transfer)
+    {
+      params->read_ref_cnt--;
+      return (sret);
+    }
 
   stpos = (char *) ptr;
   while (params->chunk_size == 0)
     {
-      current_size = retval - (stpos - (char *) ptr);
-      if (current_size < 5)
+      current_size = sret - (stpos - (char *) ptr);
+      if (current_size == 0)
+        break;
+
+      if (*stpos == '\r' && *(stpos + 1) == '\n')
+        crlfpos = strstr (stpos + 2, "\r\n");
+      else
+        crlfpos = NULL;
+
+      if (crlfpos == NULL)
         {
           /* The current chunk size is split between multiple packets.
              Save this chunk and read the next */
@@ -802,65 +807,76 @@ rfc2068_chunked_read (gftp_request * request, void *ptr, size_t size, int fd)
           memcpy (params->extra_read_buffer, stpos, current_size);
           params->extra_read_buffer[current_size] = '\0';
           params->extra_read_buffer_len = current_size;
-          retval -= current_size;
+          sret -= current_size;
 
-          if (retval == 0)
-            return (rfc2068_chunked_read (request, ptr, size, fd));
-          else
-            return (retval);
-        }
+          if (sret == 0)
+            {
+              /* Don't let a hostile web server send us in an infinite recursive
+                 loop */
 
-      if (*stpos != '\r' || *(stpos + 1) != '\n')
-        {
-          request->logging_function (gftp_logging_recv, request,
-                                     _("Received wrong response from server, disconnecting\nExpecting a carriage return and line feed before the chunk size in the server response\n"));
-          gftp_disconnect (request);
-          return (GFTP_EFATAL);
-        }
+              if (params->read_ref_cnt > 2)
+                {
+                  request->logging_function (gftp_logging_error, request, _("Received wrong response from server, disconnecting\n"));
+                  gftp_disconnect (request);
+                  params->read_ref_cnt--;
+                  return (GFTP_EFATAL);
+                }
 
-      if ((crlfpos = strstr (stpos + 2, "\r\n")) == NULL)
-        {
-          request->logging_function (gftp_logging_recv, request,
-                                     _("Received wrong response from server, disconnecting\nExpecting a carriage return and line feed after the chunk size in the server response\n"));
-          gftp_disconnect (request);
-          return (GFTP_EFATAL);
+              retval = rfc2068_chunked_read (request, ptr, size, fd);
+              if (retval < 0)
+                return (retval);
+
+              sret = retval;
+            }
+
+          params->read_ref_cnt--;
+          return (sret);
         }
 
       *crlfpos = '\0';
       crlfpos++; /* advance to line feed */
 
-      if (sscanf (stpos + 2, "%lx", &params->chunk_size) != 1)
+      if (sscanf (stpos + 2, GFTP_OFF_T_HEX_PRINTF_MOD,
+                  &params->chunk_size) != 1)
         {
           request->logging_function (gftp_logging_recv, request,
                                      _("Received wrong response from server, disconnecting\nInvalid chunk size '%s' returned by the remote server\n"), 
                                      stpos + 2);
           gftp_disconnect (request);
+          params->read_ref_cnt--;
           return (GFTP_EFATAL);
         }
 
-      retval -= crlfpos - (char *) stpos + 1;
-      current_size -= crlfpos - (char *) stpos + 1;
+      crlfsize = crlfpos - stpos + 1;
+      sret -= crlfsize;
+      current_size -= crlfsize;
 
       if (params->chunk_size == 0)
         {
           if (params->eof)
-            return (0);
+            {
+              params->read_ref_cnt--;
+              return (0);
+            }
 
           params->eof = 1;
-          return (retval);
+          params->read_ref_cnt--;
+          return (sret);
         }
 
-      memmove (stpos, crlfpos + 1, current_size);
+      memmove (stpos, crlfpos + 1, current_size + 1);
 
-      params->chunk_size -= retval;
-      if (params->chunk_size < 0)
+      if (params->chunk_size < current_size)
         {
-          stpos += retval + params->chunk_size; /* chunk size is negative */
+          stpos += params->chunk_size;
           params->chunk_size = 0;
         }
+      else
+        params->chunk_size -= current_size;
     }
 
-  return (retval);
+  params->read_ref_cnt--;
+  return (sret);
 }
 
 
@@ -880,6 +896,7 @@ rfc2068_init (gftp_request * request)
 
   request->protonum = GFTP_HTTP_NUM;
   request->init = rfc2068_init;
+  request->copy_param_options = NULL;
   request->read_function = rfc2068_chunked_read;
   request->write_function = gftp_fd_write;
   request->destroy = rfc2068_destroy;
@@ -893,8 +910,10 @@ rfc2068_init (gftp_request * request)
   request->put_next_file_chunk = NULL;
   request->end_transfer = rfc2068_end_transfer;
   request->abort_transfer = rfc2068_end_transfer; /* NOTE: uses end_transfer */
+  request->stat_filename = NULL;
   request->list_files = rfc2068_list_files;
   request->get_next_file = rfc2068_get_next_file;
+  request->get_next_dirlist_line = NULL;
   request->get_file_size = rfc2068_get_file_size;
   request->chdir = rfc2068_chdir;
   request->rmdir = NULL;
@@ -908,9 +927,9 @@ rfc2068_init (gftp_request * request)
   request->set_config_options = rfc2068_set_config_options;
   request->url_prefix = g_strdup ("http");
   request->need_hostport = 1;
-  request->need_userpass = 0;
+  request->need_username = 0;
+  request->need_password = 0;
   request->use_cache = 1;
-  request->use_threads = 1;
   request->always_connected = 1;
 
   request->protocol_data = g_malloc0 (sizeof (rfc2068_params));
