@@ -20,7 +20,7 @@
 #include "gftp.h"
 #include "ftpcommon.h"
 
-static const char cvsid[] = "$Id: rfc959.c,v 1.66 2004/03/26 21:03:50 masneyb Exp $";
+static const char cvsid[] = "$Id: rfc959.c,v 1.93 2005/01/25 02:34:18 masneyb Exp $";
 
 static gftp_textcomboedt_data gftp_proxy_type[] = {
   {N_("none"), "", 0},
@@ -32,7 +32,7 @@ static gftp_textcomboedt_data gftp_proxy_type[] = {
   {N_("user@host NOAUTH"), "USER %hu@%hh\nPASS %hp\n", 0},
   {N_("HTTP Proxy"), "http", 0},
   {N_("Custom"), "", GFTP_TEXTCOMBOEDT_EDITABLE},
-  {NULL, NULL}
+  {NULL, NULL, 0}
 };
 
 static gftp_config_vars config_vars[] = 
@@ -66,6 +66,11 @@ static gftp_config_vars config_vars[] =
    N_("This specifies how your proxy server expects us to log in. You can specify a 2 character replacement string prefixed by a % that will be replaced with the proper data. The first character can be either p for proxy or h for the host of the FTP server. The second character can be u (user), p (pass), h (host), o (port) or a (account). For example, to specify the proxy user, you can you type in %pu"), 
    GFTP_PORT_ALL, NULL},
 
+  {"ignore_pasv_address", N_("Ignore PASV address"), 
+   gftp_option_type_checkbox, GINT_TO_POINTER(0), NULL, 
+   GFTP_CVARS_FLAGS_SHOW_BOOKMARK,
+   N_("If this is enabled, then the remote FTP server's PASV IP address field will be ignored and the host's IP address will be used instead. This is often needed for routers giving their internal rather then their external IP address in a PASV reply."),
+   GFTP_PORT_ALL, NULL},
   {"passive_transfer", N_("Passive file transfers"), 
    gftp_option_type_checkbox, GINT_TO_POINTER(1), NULL, 
    GFTP_CVARS_FLAGS_SHOW_BOOKMARK,
@@ -353,8 +358,8 @@ rfc959_getcwd (gftp_request * request)
 static int
 rfc959_chdir (gftp_request * request, const char *directory)
 {
-  char ret, *tempstr;
-  int r;
+  char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
@@ -375,8 +380,8 @@ rfc959_chdir (gftp_request * request, const char *directory)
 
   if (directory != request->directory)
     {
-      if ((r = rfc959_getcwd (request)) < 0)
-        return (r);
+      if ((ret = rfc959_getcwd (request)) < 0)
+        return (ret);
     }
 
   return (0);
@@ -386,12 +391,14 @@ rfc959_chdir (gftp_request * request, const char *directory)
 static int
 rfc959_syst (gftp_request * request)
 {
+  rfc959_parms * parms;
   char *stpos, *endpos;
   int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
+  parms = request->protocol_data;
   ret = rfc959_send_command (request, "SYST\r\n", 1);
 
   if (ret < 0)
@@ -408,6 +415,7 @@ rfc959_syst (gftp_request * request)
     return (GFTP_ERETRYABLE);
 
   *endpos = '\0';
+
   if (strcmp (stpos, "UNIX") == 0)
     request->server_type = GFTP_DIRTYPE_UNIX;
   else if (strcmp (stpos, "VMS") == 0)
@@ -430,9 +438,9 @@ int
 rfc959_connect (gftp_request * request)
 {
   char tempchar, *startpos, *endpos, *tempstr, *email, *proxy_hostname, *utf8;
-  int ret, resp;
   intptr_t ascii_transfers, proxy_port;
   rfc959_parms * parms;
+  int ret, resp;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->hostname != NULL, GFTP_EFATAL);
@@ -446,9 +454,9 @@ rfc959_connect (gftp_request * request)
   gftp_lookup_request_option (request, "ftp_proxy_port", &proxy_port);
 
   if (request->username == NULL || *request->username == '\0')
-    gftp_set_username (request, "anonymous");
+    gftp_set_username (request, GFTP_ANONYMOUS_USER);
 
-  if (strcasecmp (request->username, "anonymous") == 0 &&
+  if (strcasecmp (request->username, GFTP_ANONYMOUS_USER) == 0 &&
       (request->password == NULL || *request->password == '\0'))
     {
       gftp_lookup_request_option (request, "email", &email);
@@ -462,7 +470,10 @@ rfc959_connect (gftp_request * request)
   if ((ret = rfc959_read_response (request, 1)) != '2')
     {
       gftp_disconnect (request);
-      return (ret);
+      if (ret < 0)
+        return (ret);
+      else
+        return (GFTP_ERETRYABLE);
     }
 
   if (parms->auth_tls_start != NULL)
@@ -523,7 +534,7 @@ rfc959_connect (gftp_request * request)
             return (resp);
         }
 
-      if (resp == '3' && request->account)
+      if (resp == '3' && request->account != NULL)
 	{
 	  tempstr = g_strconcat ("ACCT ", request->account, "\r\n", NULL);
 	  resp = rfc959_send_command (request, tempstr, 1);
@@ -611,14 +622,15 @@ rfc959_disconnect (gftp_request * request)
 static int
 rfc959_ipv4_data_connection_new (gftp_request * request)
 {
-  char *pos, *pos1, resp, *command;
   struct sockaddr_in data_addr;
-  int i;
+  intptr_t ignore_pasv_address;
+  char *pos, *pos1, *command;
   intptr_t passive_transfer;
   rfc959_parms * parms;
   socklen_t data_addr_len;
   unsigned int temp[6];
   unsigned char ad[6];
+  int i, resp;
 
   parms = request->protocol_data;
 
@@ -683,8 +695,30 @@ rfc959_ipv4_data_connection_new (gftp_request * request)
       for (i = 0; i < 6; i++)
         ad[i] = (unsigned char) (temp[i] & 0xff);
 
-      memcpy (&data_addr.sin_addr, &ad[0], 4);
       memcpy (&data_addr.sin_port, &ad[4], 2);
+
+      gftp_lookup_request_option (request, "ignore_pasv_address",
+                                  &ignore_pasv_address);
+      if (ignore_pasv_address)
+	{
+#if defined (HAVE_GETADDRINFO)
+          memcpy (&data_addr.sin_addr,
+                  &((struct sockaddr_in *) request->current_hostp->ai_addr)->sin_addr,
+                  sizeof (data_addr.sin_addr));
+#else
+          memcpy (&data_addr.sin_addr, request->hostp->h_addr_list[request->curhost],
+                  request->hostp->h_length);
+#endif
+
+          pos = (char *) &data_addr.sin_addr;
+          request->logging_function (gftp_logging_error, request,
+               _("Ignoring IP address in PASV response, connecting to %d.%d.%d.%d:%d\n"),
+               pos[0] & 0xff, pos[1] & 0xff, pos[2] & 0xff, pos[3] & 0xff,
+               ntohs (data_addr.sin_port));
+        }
+      else
+        memcpy (&data_addr.sin_addr, &ad[0], 4);
+
       if (connect (parms->data_connection, (struct sockaddr *) &data_addr, 
                    data_addr_len) == -1)
         {
@@ -768,12 +802,13 @@ rfc959_ipv4_data_connection_new (gftp_request * request)
 static int
 rfc959_ipv6_data_connection_new (gftp_request * request)
 {
-  char *pos, resp, buf[64], *command;
   struct sockaddr_in6 data_addr;
-  int passive_transfer;
+  char *pos, buf[64], *command;
+  intptr_t passive_transfer;
   socklen_t data_addr_len;
   rfc959_parms * parms;
   unsigned int port;
+  int resp;
 
   parms = request->protocol_data;
   if ((parms->data_connection = socket (AF_INET6, SOCK_STREAM, IPPROTO_TCP)) < 0)
@@ -796,7 +831,7 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
 
   data_addr_len = sizeof (data_addr);
   /* This condition shouldn't happen. We better check anyway... */
-  if (data_addr_len != request->hostp->ai_addrlen) 
+  if (data_addr_len != request->current_hostp->ai_addrlen) 
     {
       request->logging_function (gftp_logging_error, request,
 				 _("Error: It doesn't look like we are connected via IPv6. Aborting connection.\n"));
@@ -823,9 +858,8 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
       pos = request->last_ftp_response + 4;
       while (*pos != '(' && *pos != '\0')
         pos++;
-      pos++;
 
-      if (*pos == '\0')
+      if (*pos == '\0' || *(pos + 1) == '\0')
         {
           request->logging_function (gftp_logging_error, request,
                       _("Invalid EPSV response '%s'\n"),
@@ -834,7 +868,7 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
           return (GFTP_EFATAL);
         }
 
-      if (sscanf (pos, "|||%u|", &port) != 1)
+      if (sscanf (pos + 1, "|||%u|", &port) != 1)
         {
           request->logging_function (gftp_logging_error, request,
                       _("Invalid EPSV response '%s'\n"),
@@ -843,7 +877,7 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
           return (GFTP_EFATAL);
         }
 
-      memcpy (&data_addr, request->hostp->ai_addr, data_addr_len);
+      memcpy (&data_addr, request->current_hostp->ai_addr, data_addr_len);
       data_addr.sin6_port = htons (port);
 
       if (connect (parms->data_connection, (struct sockaddr *) &data_addr, 
@@ -858,7 +892,7 @@ rfc959_ipv6_data_connection_new (gftp_request * request)
     }
   else
     {
-      memcpy (&data_addr, request->hostp->ai_addr, data_addr_len);
+      memcpy (&data_addr, request->current_hostp->ai_addr, data_addr_len);
       data_addr.sin6_port = 0;
 
       if (bind (parms->data_connection, (struct sockaddr *) &data_addr, 
@@ -928,7 +962,7 @@ rfc959_data_connection_new (gftp_request * request)
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
 #ifdef HAVE_IPV6
-  if (GFTP_GET_AI_FAMILY(request) == AF_INET6)
+  if (request->ai_family == AF_INET6)
     return (rfc959_ipv6_data_connection_new (request));
   else
     return (rfc959_ipv4_data_connection_new (request));
@@ -989,9 +1023,9 @@ rfc959_is_ascii_transfer (gftp_request * request, const char *filename)
 {
   gftp_config_list_vars * tmplistvar;
   gftp_file_extensions * tempext;
-  int stlen;
   intptr_t ascii_transfers;
   GList * templist;
+  size_t stlen;
   
   gftp_lookup_global_option ("ext", &tmplistvar);
   gftp_lookup_request_option (request, "ascii_transfers", &ascii_transfers);
@@ -1053,10 +1087,10 @@ static off_t
 rfc959_get_file (gftp_request * request, const char *filename, int fd,
                  off_t startsize)
 {
-  char *command, *tempstr, resp;
-  int ret;
+  char *command, *tempstr;
   intptr_t passive_transfer;
   rfc959_parms * parms;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (filename != NULL, GFTP_EFATAL);
@@ -1078,17 +1112,14 @@ rfc959_get_file (gftp_request * request, const char *filename, int fd,
 
   if (startsize > 0)
     {
-#if defined (_LARGEFILE_SOURCE)
-      command = g_strdup_printf ("REST %lld\r\n", (long long) startsize); 
-#else
-      command = g_strdup_printf ("REST %ld\r\n", startsize); 
-#endif
-      resp = rfc959_send_command (request, command, 1);
+      command = g_strdup_printf ("REST " GFTP_OFF_T_PRINTF_MOD "\r\n",
+                                 startsize);
+      ret = rfc959_send_command (request, command, 1);
       g_free (command);
 
-      if (resp < 0)
-        return (resp);
-      else if (resp != '3')
+      if (ret < 0)
+        return (ret);
+      else if (ret != '3')
         {
           close (parms->data_connection);
           parms->data_connection = -1;
@@ -1131,8 +1162,8 @@ static int
 rfc959_put_file (gftp_request * request, const char *filename, int fd,
                  off_t startsize, off_t totalsize)
 {
-  char *command, *tempstr, resp;
   intptr_t passive_transfer;
+  char *command, *tempstr;
   rfc959_parms * parms;
   int ret;
 
@@ -1156,16 +1187,13 @@ rfc959_put_file (gftp_request * request, const char *filename, int fd,
 
   if (startsize > 0)
     {
-#if defined (_LARGEFILE_SOURCE)
-      command = g_strdup_printf ("REST %lld\r\n", (long long) startsize); 
-#else
-      command = g_strdup_printf ("REST %ld\r\n", startsize); 
-#endif
-      resp = rfc959_send_command (request, command, 1);
+      command = g_strdup_printf ("REST " GFTP_OFF_T_PRINTF_MOD "\r\n",
+                                 startsize);
+      ret = rfc959_send_command (request, command, 1);
       g_free (command);
-      if (resp < 0)
-        return (resp);
-      else if (resp != '3')
+      if (ret < 0)
+        return (ret);
+      else if (ret != '3')
         {
           close (parms->data_connection);
           parms->data_connection = -1;
@@ -1328,9 +1356,9 @@ rfc959_abort_transfer (gftp_request * request)
 static int
 rfc959_list_files (gftp_request * request)
 {
-  int ret;
   intptr_t show_hidden_files, resolve_symlinks, passive_transfer;
   char *tempstr, parms[3];
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
@@ -1404,10 +1432,9 @@ static ssize_t
 rfc959_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 {
   rfc959_parms * parms;
+  size_t rsize, i, j;
   ssize_t num_wrote;
   char *tempstr;
-  size_t rsize;
-  int i, j;
 
   if (size == 0)
     return (0);
@@ -1457,13 +1484,31 @@ rfc959_put_next_file_chunk (gftp_request * request, char *buf, size_t size)
 }
 
 
-int
-rfc959_get_next_file (gftp_request * request, gftp_file * fle, int fd)
+static ssize_t
+rfc959_get_next_dirlist_line (gftp_request * request, int fd,
+                              char *buf, size_t buflen)
 {
   ssize_t (*oldread_func) (gftp_request * request, void *ptr, size_t size,
                            int fd);
   rfc959_parms * parms;
+  ssize_t len;
+
+  parms = request->protocol_data;
+
+  oldread_func = request->read_function;
+  request->read_function = parms->data_conn_read;
+  len = gftp_get_line (request, &parms->dataconn_rbuf, buf, buflen, fd);
+  request->read_function = oldread_func;
+
+  return (len);
+}
+
+int
+rfc959_get_next_file (gftp_request * request, gftp_file * fle, int fd)
+{
+  rfc959_parms * parms;
   char tempstr[1024];
+  size_t stlen;
   ssize_t len;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
@@ -1483,26 +1528,22 @@ rfc959_get_next_file (gftp_request * request, gftp_file * fle, int fd)
 
   do
     {
-      oldread_func = request->read_function;
-      request->read_function = parms->data_conn_read;
-      len = gftp_get_line (request, &parms->dataconn_rbuf, tempstr,
-                           sizeof (tempstr), fd);
-      request->read_function = oldread_func;
-
+      len = rfc959_get_next_dirlist_line (request, fd, tempstr,
+                                          sizeof (tempstr));
       if (len <= 0)
 	{
-          gftp_file_destroy (fle);
-	  return ((int) len);
+          gftp_file_destroy (fle, 0);
+	  return (len);
 	} 
 
-      if (gftp_parse_ls (request, tempstr, fle) != 0)
+      if (gftp_parse_ls (request, tempstr, fle, fd) != 0)
 	{
 	  if (strncmp (tempstr, "total", strlen ("total")) != 0 &&
 	      strncmp (tempstr, _("total"), strlen (_("total"))) != 0)
 	    request->logging_function (gftp_logging_error, request,
 				       _("Warning: Cannot parse listing %s\n"),
 				       tempstr);
-	  gftp_file_destroy (fle);
+	  gftp_file_destroy (fle, 0);
 	  continue;
 	}
       else
@@ -1510,11 +1551,11 @@ rfc959_get_next_file (gftp_request * request, gftp_file * fle, int fd)
     }
   while (1);
 
-  len = strlen (tempstr);
+  stlen = strlen (tempstr);
   if (!request->cached)
     {
       request->last_dir_entry = g_strdup_printf ("%s\n", tempstr);
-      request->last_dir_entry_len = strlen (tempstr) + 1;
+      request->last_dir_entry_len = stlen + 1;
     }
   return (len);
 }
@@ -1545,7 +1586,8 @@ rfc959_get_file_size (gftp_request * request, const char *filename)
 static int
 rfc959_rmdir (gftp_request * request, const char *directory)
 {
-  char *tempstr, ret;
+  char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
@@ -1567,7 +1609,8 @@ rfc959_rmdir (gftp_request * request, const char *directory)
 static int
 rfc959_rmfile (gftp_request * request, const char *file)
 {
-  char *tempstr, ret;
+  char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
@@ -1589,7 +1632,8 @@ rfc959_rmfile (gftp_request * request, const char *file)
 static int
 rfc959_mkdir (gftp_request * request, const char *directory)
 {
-  char *tempstr, ret;
+  char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (directory != NULL, GFTP_EFATAL);
@@ -1612,7 +1656,8 @@ static int
 rfc959_rename (gftp_request * request, const char *oldname,
 	       const char *newname)
 {
-  char *tempstr, ret;
+  char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (oldname != NULL, GFTP_EFATAL);
@@ -1642,16 +1687,19 @@ rfc959_rename (gftp_request * request, const char *oldname,
 
 
 static int
-rfc959_chmod (gftp_request * request, const char *file, int mode)
+rfc959_chmod (gftp_request * request, const char *file, mode_t mode)
 {
-  char *tempstr, ret;
+  rfc959_parms * parms;
+  char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (file != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  tempstr = g_malloc (strlen (file) + (mode / 10) + 16);
-  sprintf (tempstr, "SITE CHMOD %d %s\r\n", mode, file);
+  parms = request->protocol_data;
+  tempstr = g_strdup_printf ("SITE CHMOD %o %s\r\n", mode, file);
+
   ret = rfc959_send_command (request, tempstr, 1);
   g_free (tempstr);
 
@@ -1665,15 +1713,76 @@ rfc959_chmod (gftp_request * request, const char *file, int mode)
 
 
 static int
-rfc959_site (gftp_request * request, const char *command)
+rfc959_site (gftp_request * request, int specify_site, const char *command)
 {
-  char *tempstr, ret;
+  char *tempstr;
+  int ret;
 
   g_return_val_if_fail (request != NULL, GFTP_EFATAL);
   g_return_val_if_fail (command != NULL, GFTP_EFATAL);
   g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
 
-  tempstr = g_strconcat ("SITE ", command, "\r\n", NULL);
+  if (specify_site)
+    tempstr = g_strconcat ("SITE ", command, "\r\n", NULL);
+  else
+    tempstr = g_strconcat (command, "\r\n", NULL);
+
+  ret = rfc959_send_command (request, tempstr, 1);
+  g_free (tempstr);
+
+  if (ret < 0)
+    return (ret);
+  else if (ret == '2')
+    return (0);
+  else
+    return (GFTP_ERETRYABLE);
+}
+
+
+static char *
+rfc959_time_t_to_mdtm (gftp_request * request, time_t datetime)
+{
+  struct tm gt;
+  char *ret;
+
+  if (localtime_r (&datetime, &gt) != NULL)
+    {
+      ret = g_strdup_printf ("%04d%02d%02d%02d%02d%02d", gt.tm_year + 1900,
+                             gt.tm_mon + 1, gt.tm_mday, gt.tm_hour, gt.tm_min,
+                             gt.tm_sec);
+      return (ret);
+    }
+  else
+    {
+      request->logging_function (gftp_logging_error, request,
+                                 "Cannot parse UNIX timestamp %d: %s\n",
+                                 datetime, g_strerror (errno));
+      return (NULL);
+    }
+}
+
+
+static int
+rfc959_set_file_time (gftp_request * request, const char *file, time_t datetime)
+{
+  char *tempstr, *datestr;
+  rfc959_parms * parms;
+  int ret;
+
+  g_return_val_if_fail (request != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (file != NULL, GFTP_EFATAL);
+  g_return_val_if_fail (request->datafd > 0, GFTP_EFATAL);
+
+  parms = request->protocol_data;
+
+  datestr = rfc959_time_t_to_mdtm (request, datetime);
+  if (datestr == NULL)
+    return (GFTP_EFATAL);
+
+  tempstr = g_strconcat ("SITE UTIME ", datestr, " ", file, "\r\n", NULL);
+
+  g_free (datestr);
+
   ret = rfc959_send_command (request, tempstr, 1);
   g_free (tempstr);
 
@@ -1712,7 +1821,7 @@ rfc959_register_module (void)
 }
 
 
-void
+static void
 rfc959_request_destroy (gftp_request * request)
 {
   rfc959_parms * parms;
@@ -1727,7 +1836,7 @@ rfc959_request_destroy (gftp_request * request)
 }
 
 
-void
+static void
 rfc959_copy_param_options (gftp_request * dest_request,
                            gftp_request * src_request)
 {
@@ -1791,8 +1900,10 @@ rfc959_init (gftp_request * request)
   request->put_next_file_chunk = rfc959_put_next_file_chunk;
   request->end_transfer = rfc959_end_transfer;
   request->abort_transfer = rfc959_abort_transfer;
+  request->stat_filename = NULL;
   request->list_files = rfc959_list_files;
   request->get_next_file = rfc959_get_next_file;
+  request->get_next_dirlist_line = rfc959_get_next_dirlist_line;
   request->get_file_size = rfc959_get_file_size;
   request->chdir = rfc959_chdir;
   request->rmdir = rfc959_rmdir;
@@ -1800,14 +1911,15 @@ rfc959_init (gftp_request * request)
   request->mkdir = rfc959_mkdir;
   request->rename = rfc959_rename;
   request->chmod = rfc959_chmod;
-  request->set_file_time = NULL;
+  request->set_file_time = rfc959_set_file_time;
   request->site = rfc959_site;
   request->parse_url = NULL;
   request->swap_socks = NULL;
   request->set_config_options = rfc959_set_config_options;
   request->url_prefix = "ftp";
   request->need_hostport = 1;
-  request->need_userpass = 1;
+  request->need_username = 1;
+  request->need_password = 1;
   request->use_cache = 1;
   request->always_connected = 0;
 
