@@ -18,12 +18,12 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: bookmarks.c,v 1.27 2004/03/28 23:58:13 masneyb Exp $";
+static const char cvsid[] = "$Id: bookmarks.c,v 1.40 2004/11/05 02:11:59 masneyb Exp $";
 
-static GtkWidget * bm_dialog = NULL, * bm_hostedit, * bm_portedit,
-                 * bm_localdiredit, * bm_remotediredit, * bm_useredit,
-                 * bm_passedit, * bm_acctedit, * anon_chk, * bm_pathedit,
-                 * bm_protocol, * tree, * edit_bookmarks_dialog;
+static GtkWidget * bm_dialog = NULL, * edit_bookmarks_dialog = NULL;
+static GtkWidget * bm_hostedit, * bm_portedit, * bm_localdiredit,
+                 * bm_remotediredit, * bm_useredit, * bm_passedit, * tree,
+                 * bm_acctedit, * anon_chk, * bm_pathedit, * bm_protocol;
 static GHashTable * new_bookmarks_htable = NULL;
 static gftp_bookmarks_var * new_bookmarks = NULL;
 static GtkItemFactory * edit_factory;
@@ -50,7 +50,7 @@ run_bookmark (gpointer data)
     return;
 
   if (refresh_local)
-    gftpui_refresh (other_wdata);
+    gftpui_refresh (other_wdata, 0);
 
   ftp_connect (current_wdata, current_wdata->request, 1);
 }
@@ -59,7 +59,7 @@ run_bookmark (gpointer data)
 static void
 doadd_bookmark (gpointer * data, gftp_dialog_data * ddata)
 {
-  GtkItemFactoryEntry test = { NULL, NULL, run_bookmark, 0 };
+  GtkItemFactoryEntry test = { NULL, NULL, run_bookmark, 0, MN_(NULL) };
   const char *edttxt, *spos;
   gftp_bookmarks_var * tempentry;
   char *dpos, *proto;
@@ -150,7 +150,7 @@ add_bookmark (gpointer data)
 void
 build_bookmarks_menu (void)
 {
-  GtkItemFactoryEntry test = { NULL, NULL, NULL, 0 };
+  GtkItemFactoryEntry test = { NULL, NULL, NULL, 0, MN_(NULL) };
   gftp_bookmarks_var * tempentry;
 
   tempentry = gftp_bookmarks->children;
@@ -167,6 +167,7 @@ build_bookmarks_menu (void)
           test.item_type = "";
           test.callback = run_bookmark;
         }
+
       gtk_item_factory_create_item (factory, &test,
                                     (gpointer) tempentry->path, 1);
       g_free (test.path);
@@ -280,82 +281,79 @@ copy_bookmarks (gftp_bookmarks_var * bookmarks)
 }
 
 
+static void
+_free_menu_entry (gftp_bookmarks_var * entry)
+{
+  GtkWidget * tempwid;
+  char *tempstr;
+
+  if (entry->oldpath != NULL)
+    tempstr = gftp_build_path (NULL, "/Bookmarks", entry->oldpath, NULL);
+  else
+    tempstr = gftp_build_path (NULL, "/Bookmarks", entry->path, NULL);
+
+  tempwid = gtk_item_factory_get_item (factory, tempstr);
+  if (GTK_IS_WIDGET (tempwid))
+    gtk_widget_destroy (tempwid);
+
+  g_free (tempstr);
+}
+
+
 static void
 bm_apply_changes (GtkWidget * widget, gpointer backup_data)
 {
   gftp_bookmarks_var * tempentry, * delentry;
-  char *tempstr;
 
-  tempentry = gftp_bookmarks->children;
-  while (tempentry != NULL)
+  if (bm_dialog != NULL)
     {
-      if (tempentry->path && !tempentry->isfolder)
-	{
-	  tempstr = g_strdup_printf ("/Bookmarks/%s", tempentry->path);
-          gtk_widget_destroy (gtk_item_factory_get_item (factory, tempstr));
-	  g_free (tempentry->path);
-	  g_free (tempstr);
-	}
+      gtk_widget_grab_focus (bm_dialog);
+      return;
+    }
 
-      gftp_free_bookmark (tempentry);
+  if (gftp_bookmarks != NULL)
+    {
+      tempentry = gftp_bookmarks->children;
+      while (tempentry != NULL)
+        {
+          if (tempentry->children != NULL)
+            tempentry = tempentry->children;
+          else
+            {
+              while (tempentry->next == NULL && tempentry->prev != NULL)
+                {
+                  delentry = tempentry;
+                  tempentry = tempentry->prev;
+                  _free_menu_entry (delentry);
+                  gftp_free_bookmark (delentry, 1);
+                }
 
-      if (tempentry->children != NULL)
-	tempentry = tempentry->children;
-      else
-	{
-	  if (tempentry->next == NULL)
-	    {
-	      while (tempentry->next == NULL && tempentry->prev != NULL)
-		{
-		  delentry = tempentry;
-		  tempentry = tempentry->prev;
-		  if (delentry->isfolder)
-		    {
-		      tempstr = g_strdup_printf ("/Bookmarks/%s", 
-                                                 delentry->path);
-                      gtk_widget_destroy (gtk_item_factory_get_item (factory, 
-                                                                     tempstr));
-		      g_free (tempstr);
-		      g_free (delentry->path);
-		    }
-		  g_free (delentry);
-		}
-	      delentry = tempentry;
-	      tempentry = tempentry->next;
-	      if (delentry->isfolder && tempentry != NULL)
-		{
-		  tempstr = g_strdup_printf ("/Bookmarks/%s", 
-                                             delentry->path);
-                    gtk_widget_destroy (gtk_item_factory_get_item (factory, 
-                                                                   tempstr));
-		  g_free (delentry->path);
-		  g_free (tempstr);
-		  g_free (delentry);
-		}
-	    }
-	  else
-	    {
-	      delentry = tempentry;
-	      tempentry = tempentry->next;
-	      g_free (delentry);
-	    }
-	}
+              delentry = tempentry;
+              tempentry = tempentry->next;
+              if (tempentry != NULL)
+                _free_menu_entry (delentry);
+
+              gftp_free_bookmark (delentry, 1);
+            }
+        }
+
+      g_hash_table_destroy (gftp_bookmarks_htable);
     }
-  g_free (gftp_bookmarks);
-  g_hash_table_destroy (gftp_bookmarks_htable);
 
-  gftp_bookmarks = new_bookmarks;
-  gftp_bookmarks_htable = new_bookmarks_htable;
   if (backup_data)
     {
-      new_bookmarks = copy_bookmarks (gftp_bookmarks);
-      new_bookmarks_htable = build_bookmarks_hash_table (new_bookmarks);
+      gftp_bookmarks = copy_bookmarks (new_bookmarks);
+      gftp_bookmarks_htable = build_bookmarks_hash_table (gftp_bookmarks);
     }
   else
     {
+      gftp_bookmarks = new_bookmarks;
+      gftp_bookmarks_htable = new_bookmarks_htable;
+
       new_bookmarks = NULL;
       new_bookmarks_htable = NULL;
     }
+
   build_bookmarks_menu ();
   gftp_write_bookmarks_file ();
 }
@@ -364,6 +362,9 @@ bm_apply_changes (GtkWidget * widget, gpointer backup_data)
 static void
 bm_close_dialog (GtkWidget * widget, GtkWidget * dialog)
 {
+  if (bm_dialog != NULL)
+    return;
+
   if (new_bookmarks_htable != NULL)
     {
       g_hash_table_destroy (new_bookmarks_htable);
@@ -376,8 +377,11 @@ bm_close_dialog (GtkWidget * widget, GtkWidget * dialog)
       new_bookmarks = NULL;
     }
 
-  gtk_widget_destroy (edit_bookmarks_dialog);
-  edit_bookmarks_dialog = NULL;
+  if (edit_bookmarks_dialog != NULL)
+    {
+      gtk_widget_destroy (edit_bookmarks_dialog);
+      edit_bookmarks_dialog = NULL;
+    }
 }
 
 
@@ -403,7 +407,6 @@ do_make_new (gpointer data, gftp_dialog_data * ddata)
   GdkPixmap * closedir_pixmap, * opendir_pixmap;
   GdkBitmap * closedir_bitmap, * opendir_bitmap;  
   gftp_bookmarks_var * tempentry, * newentry;
-  GtkCTreeNode * sibling;
   char *pos, *text[2];
   const char *str;
 #if GTK_MAJOR_VERSION > 1
@@ -436,17 +439,13 @@ do_make_new (gpointer data, gftp_dialog_data * ddata)
     newentry->isfolder = 1;
 
   if (new_bookmarks->children == NULL)
-    {
-      new_bookmarks->children = newentry;
-      sibling = NULL;
-    }
+    new_bookmarks->children = newentry;
   else
     {
       tempentry = new_bookmarks->children;
       while (tempentry->next != NULL)
 	tempentry = tempentry->next;
       tempentry->next = newentry;
-      sibling = tempentry->cnode;
     }
 
   text[0] = text[1] = newentry->path;
@@ -461,7 +460,7 @@ do_make_new (gpointer data, gftp_dialog_data * ddata)
     newentry->cnode = gtk_ctree_insert_node (GTK_CTREE (tree), 
                                              new_bookmarks->cnode, NULL,
                                              text, 5, NULL, NULL, NULL, NULL, 
-                                             FALSE, FALSE);
+                                             TRUE, FALSE);
 
   gtk_ctree_node_set_row_data (GTK_CTREE (tree), newentry->cnode, newentry);
   g_hash_table_insert (new_bookmarks_htable, newentry->path, newentry);
@@ -510,7 +509,7 @@ do_delete_entry (gftp_bookmarks_var * entry, gftp_dialog_data * ddata)
   tempentry = entry;
   while (tempentry != NULL)
     {
-      gftp_free_bookmark (tempentry);
+      gftp_free_bookmark (tempentry, 0);
 
       if (tempentry->children != NULL)
 	{
@@ -573,102 +572,96 @@ set_userpass_visible (GtkWidget * checkbutton, GtkWidget * entry)
 
 
 static void
-build_bookmarks_tree (void)
+_add_tree_node (gftp_bookmarks_var * entry, char *path, int isleaf,
+                int expanded)
 {
-  GdkPixmap * closedir_pixmap, * opendir_pixmap;
-  GdkBitmap * closedir_bitmap, * opendir_bitmap;  
-  gftp_bookmarks_var * tempentry, * preventry;
-  char *pos, *prevpos, *text[2], *str;
+  static GdkPixmap * closedir_pixmap = NULL,
+                   * opendir_pixmap = NULL;
+  static GdkBitmap * closedir_bitmap = NULL,
+                   * opendir_bitmap = NULL;
+  static int initialized = 0;
   GtkCTreeNode * parent;
+  char *text[2], *pos;
 
-  gftp_get_pixmap (tree, "open_dir.xpm", &opendir_pixmap, &opendir_bitmap);
-  gftp_get_pixmap (tree, "dir.xpm", &closedir_pixmap, &closedir_bitmap);
-  text[0] = text[1] = _("Bookmarks");
-  parent = gtk_ctree_insert_node (GTK_CTREE (tree), NULL, NULL, text, 5, 
-                                  closedir_pixmap, closedir_bitmap, 
-                                  opendir_pixmap, opendir_bitmap, FALSE, TRUE);
-  gtk_ctree_node_set_row_data (GTK_CTREE (tree), parent, new_bookmarks);
-  new_bookmarks->cnode = parent;
-
-  tempentry = new_bookmarks->children;
-  while (tempentry != NULL)
+  if (!initialized)
     {
-      tempentry->cnode = NULL;
-      if (tempentry->children != NULL)
-	{
-	  tempentry = tempentry->children;
-	  continue;
-	}
-      else
-	{
-	  pos = tempentry->path;
-	  while ((pos = strchr (pos, '/')) != NULL)
-	    {
-	      *pos = '\0';
-              str = g_strdup (tempentry->path);
-	      *pos = '/';
-	      preventry = g_hash_table_lookup (new_bookmarks_htable, str);
-	      if (preventry->cnode == NULL)
-		{
-		  if ((prevpos = strrchr (str, '/')) == NULL)
-		    prevpos = str;
-		  else
-		    prevpos++;
-		  text[0] = text[1] = prevpos;
-		  preventry->cnode = gtk_ctree_insert_node (GTK_CTREE (tree),
-					   preventry->prev->cnode, NULL, text,
-					   5, closedir_pixmap, closedir_bitmap,
-					   opendir_pixmap, opendir_bitmap,
-					   FALSE, FALSE);
-		  gtk_ctree_node_set_row_data (GTK_CTREE (tree),
-					       preventry->cnode, preventry);
-		}
-
-	      g_free (str);
-	      pos++;
-	    }
-	}
-
-      if ((pos = strrchr (tempentry->path, '/')) == NULL)
-	pos = tempentry->path;
-      else
-	pos++;
-
-      text[0] = text[1] = pos;
-      tempentry->cnode = gtk_ctree_insert_node (GTK_CTREE (tree), 
-                               tempentry->prev->cnode, NULL,
-			       text, 5, NULL, NULL, NULL, NULL, FALSE, FALSE);
-      gtk_ctree_node_set_row_data (GTK_CTREE (tree), tempentry->cnode,
-				   tempentry);
-
-      while (tempentry->next == NULL && tempentry->prev != NULL)
-	tempentry = tempentry->prev;
-      tempentry = tempentry->next;
+      gftp_get_pixmap (tree, "open_dir.xpm", &opendir_pixmap, &opendir_bitmap);
+      gftp_get_pixmap (tree, "dir.xpm", &closedir_pixmap, &closedir_bitmap);
+      initialized = 1;
     }
+
+  if (path == NULL)
+    {
+      if ((pos = strrchr (entry->path, '/')) == NULL)
+        pos = entry->path;
+      else
+        pos++;
+    }
+  else
+    pos = path;
+
+  text[0] = text[1] = pos;
+
+  if (entry->prev == NULL)
+    parent = NULL;
+  else
+    parent = entry->prev->cnode;
+
+  if (isleaf) 
+    entry->cnode = gtk_ctree_insert_node (GTK_CTREE (tree), parent, NULL, text,
+                                          5, NULL, NULL, NULL, NULL, TRUE,
+                                          expanded);
+  else
+    entry->cnode = gtk_ctree_insert_node (GTK_CTREE (tree), parent, NULL,
+                                          text, 5, closedir_pixmap,
+                                          closedir_bitmap, opendir_pixmap,
+                                          opendir_bitmap, FALSE, expanded);
+
+  gtk_ctree_node_set_row_data (GTK_CTREE (tree), entry->cnode, entry);
 }
 
 
 static void
-clear_bookmarks_tree (void)
+build_bookmarks_tree (void)
 {
-  gftp_bookmarks_var * tempentry;
+  gftp_bookmarks_var * tempentry, * preventry;
+  char *pos, *tempstr;
+
+  _add_tree_node (new_bookmarks, _("Bookmarks"), 0, 1);
 
   tempentry = new_bookmarks->children;
   while (tempentry != NULL)
     {
+      tempentry->cnode = NULL;
+
       if (tempentry->children != NULL)
 	{
 	  tempentry = tempentry->children;
 	  continue;
 	}
-      while (tempentry->next == NULL && tempentry->prev != NULL)
-	{
-	  gtk_ctree_remove_node (GTK_CTREE (tree), tempentry->cnode);
-	  tempentry->cnode = NULL;
-	  tempentry = tempentry->prev;
+      else if (strchr (tempentry->path, '/') == NULL && tempentry->isfolder)
+        _add_tree_node (tempentry, NULL, 0, 0);
+      else
+        {
+          pos = tempentry->path;
+          while ((pos = strchr (pos, '/')) != NULL)
+            {
+              *pos = '\0';
+              tempstr = g_strdup (tempentry->path);
+              *pos++ = '/';
+
+              preventry = g_hash_table_lookup (new_bookmarks_htable, tempstr);
+              g_free (tempstr);
+
+              if (preventry->cnode == NULL)
+                _add_tree_node (preventry, NULL, 0, 0);
+            }
+
+          _add_tree_node (tempentry, NULL, TRUE, 0);
 	}
-      gtk_ctree_remove_node (GTK_CTREE (tree), tempentry->cnode);
-      tempentry->cnode = NULL;
+
+      while (tempentry->next == NULL && tempentry->prev != NULL)
+        tempentry = tempentry->prev;
       tempentry = tempentry->next;
     }
 }
@@ -677,30 +670,31 @@ clear_bookmarks_tree (void)
 static void
 entry_apply_changes (GtkWidget * widget, gftp_bookmarks_var * entry)
 {
-  char *pos, *newpath, tempchar, *tempstr, *origpath;
-  gftp_bookmarks_var * tempentry, * nextentry;
+  gftp_bookmarks_var * tempentry, * nextentry, * bmentry;
+  char *pos, *newpath, tempchar, *tempstr;
   GtkWidget * tempwid;
+  size_t oldpathlen;
   const char *str;
-  int oldpathlen;
+
+  tempstr = g_strdup (gtk_entry_get_text (GTK_ENTRY (bm_pathedit)));
+  while ((pos = strchr (tempstr, '/')) != NULL)
+    *pos = ' ';
 
   oldpathlen = strlen (entry->path);
-  if ((pos = strrchr (entry->path, '/')) == NULL)
+  if ((pos = strrchr (entry->path, '/')) != NULL)
     {
       pos = entry->path;
-      tempchar = *entry->path;
-      *entry->path = '\0';
-    }
-  else
-    {
       tempchar = *pos;
       *pos = '\0';
+      newpath = gftp_build_path (NULL, entry->path, tempstr, NULL);
+      *pos = tempchar;
+      g_free (tempstr);
     }
-
-  origpath = newpath = gftp_build_path (entry->path, gtk_entry_get_text (GTK_ENTRY (bm_pathedit)), NULL);
-  *pos = tempchar;
+  else
+    newpath = tempstr;
 
   str = gtk_entry_get_text (GTK_ENTRY (bm_hostedit));
-  if (entry->hostname)
+  if (entry->hostname != NULL)
     g_free (entry->hostname);
   entry->hostname = g_strdup (str);
 
@@ -709,42 +703,40 @@ entry_apply_changes (GtkWidget * widget, gftp_bookmarks_var * entry)
 
   tempwid = gtk_menu_get_active (GTK_MENU (bm_protocol));
   str = gtk_object_get_user_data (GTK_OBJECT (tempwid));
-  if (entry->protocol)
+  if (entry->protocol != NULL)
     g_free (entry->protocol);
   entry->protocol = g_strdup (str);
 
   str = gtk_entry_get_text (GTK_ENTRY (bm_remotediredit));
-  if (entry->remote_dir)
+  if (entry->remote_dir != NULL)
     g_free (entry->remote_dir);
   entry->remote_dir = g_strdup (str);
 
   str = gtk_entry_get_text (GTK_ENTRY (bm_localdiredit));
-  if (entry->local_dir)
+  if (entry->local_dir != NULL)
     g_free (entry->local_dir);
   entry->local_dir = g_strdup (str);
 
   if (GTK_TOGGLE_BUTTON (anon_chk)->active)
-    str = "anonymous";
+    str = GFTP_ANONYMOUS_USER;
   else
     str = gtk_entry_get_text (GTK_ENTRY (bm_useredit));
-  if (entry->user)
+
+  if (entry->user != NULL)
     g_free (entry->user);
   entry->user = g_strdup (str);
 
-  if (GTK_TOGGLE_BUTTON (anon_chk)->active)
+  if (strcasecmp (entry->user, GFTP_ANONYMOUS_USER) == 0)
     str = "@EMAIL@";
   else
     str = gtk_entry_get_text (GTK_ENTRY (bm_passedit));
-  if (entry->pass)
+  if (entry->pass != NULL)
     g_free (entry->pass);
   entry->pass = g_strdup (str);
   entry->save_password = *entry->pass != '\0';
 
-  if (GTK_TOGGLE_BUTTON (anon_chk)->active)
-    str = "";
-  else
-    str = gtk_entry_get_text (GTK_ENTRY (bm_acctedit));
-  if (entry->acct)
+  str = gtk_entry_get_text (GTK_ENTRY (bm_acctedit));
+  if (entry->acct != NULL)
     g_free (entry->acct);
   entry->acct = g_strdup (str);
 
@@ -755,36 +747,47 @@ entry_apply_changes (GtkWidget * widget, gftp_bookmarks_var * entry)
       tempentry = entry;
       nextentry = entry->next;
       entry->next = NULL;
+
       while (tempentry != NULL)
 	{
 	  g_hash_table_remove (new_bookmarks_htable, tempentry->path);
 
-          if (tempentry->path + oldpathlen == '\0')
-	    tempstr = g_strdup (newpath);
+          bmentry = g_hash_table_lookup (gftp_bookmarks_htable,
+                                         tempentry->path);
+          if (bmentry->oldpath == NULL)
+            bmentry->oldpath = tempentry->path;
           else
-	    tempstr = gftp_build_path (newpath, tempentry->path + oldpathlen, NULL);
+            g_free (tempentry->path);
+
+          if (*(tempentry->path + oldpathlen) == '\0')
+	    tempentry->path = g_strdup (newpath);
+          else
+	    tempentry->path = gftp_build_path (NULL, newpath,
+                                               tempentry->path + oldpathlen,
+                                               NULL);
 
-	  g_free (tempentry->path);
-	  tempentry->path = tempstr;
 	  g_hash_table_insert (new_bookmarks_htable, tempentry->path,
-			       tempentry);
-	  if (tempentry->children != NULL)
-	    {
-	      tempentry = tempentry->children;
-	      continue;
-	    }
-	  while (tempentry->next == NULL && tempentry != entry &&
-		 tempentry->prev != NULL)
-	    tempentry = tempentry->prev;
+                               tempentry);
 
-	  tempentry = tempentry->next;
+          gtk_ctree_node_set_text (GTK_CTREE (tree), tempentry->cnode, 0,
+                                   tempentry->path);
+
+	  if (tempentry->children != NULL)
+	    tempentry = tempentry->children;
+          else
+            {
+	      while (tempentry->next == NULL && tempentry != entry &&
+                     tempentry->prev != NULL)
+                tempentry = tempentry->prev;
+
+	      tempentry = tempentry->next;
+            }
 	}
+
       entry->next = nextentry;
-      clear_bookmarks_tree ();
-      build_bookmarks_tree ();
     }
 
-  g_free (origpath);
+  g_free (newpath);
 }
 
 
@@ -1054,8 +1057,7 @@ edit_entry (gpointer data)
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (bm_dialog)->action_area), tempwid,
 		      TRUE, TRUE, 0);
   gtk_signal_connect_object (GTK_OBJECT (tempwid), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (bm_dialog));
+			     GTK_SIGNAL_FUNC (entry_close_dialog), NULL);
   GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
   gtk_widget_grab_focus (tempwid);
   gtk_widget_show (tempwid);
@@ -1210,13 +1212,13 @@ edit_bookmarks (gpointer data)
   GtkItemFactory * ifactory;
   GtkWidget * scroll;
   GtkItemFactoryEntry menu_items[] = {
-    {N_("/_File"), NULL, 0, 0, "<Branch>"},
-    {N_("/File/tearoff"), NULL, 0, 0, "<Tearoff>"},
+    {N_("/_File"), NULL, 0, 0, MN_("<Branch>")},
+    {N_("/File/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
     {N_("/File/New Folder..."), NULL, new_folder_entry, 0, MN_(NULL)},
     {N_("/File/New Item..."), NULL, new_item_entry, 0, MS_(GTK_STOCK_NEW)},
     {N_("/File/Delete"), NULL, delete_entry, 0, MS_(GTK_STOCK_DELETE)},
     {N_("/File/Properties..."), NULL, edit_entry, 0, MS_(GTK_STOCK_PROPERTIES)},
-    {N_("/File/sep"), NULL, 0, 0, "<Separator>"},
+    {N_("/File/sep"), NULL, 0, 0, MN_("<Separator>")},
     {N_("/File/Close"), NULL, gtk_widget_destroy, 0, MS_(GTK_STOCK_CLOSE)}
   };
 #if GTK_MAJOR_VERSION == 1
