@@ -18,15 +18,17 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-static const char cvsid[] = "$Id: gftp-gtk.c,v 1.45 2004/03/21 23:56:18 masneyb Exp $";
+static const char cvsid[] = "$Id: gftp-gtk.c,v 1.66 2005/01/25 02:34:19 masneyb Exp $";
 
 static GtkItemFactory *log_factory, *dl_factory;
 static GtkWidget * local_frame, * remote_frame, * log_table, * transfer_scroll,
-                 * openurl_btn, * gftpui_command_toolbar;
+                 * gftpui_command_toolbar;
 
 gftp_window_data window1, window2, *other_wdata, *current_wdata;
 GtkWidget * stop_btn, * hostedit, * useredit, * passedit, * portedit, * logwdw,
-          * dlwdw, * protocol_menu, * optionmenu, * gftpui_command_widget;
+          * dlwdw, * protocol_menu, * optionmenu, * gftpui_command_widget,
+          * download_left_arrow, * upload_right_arrow, * openurl_btn;
+GtkTooltips * openurl_tooltip;
 GtkAdjustment * logwdw_vadj;
 #if GTK_MAJOR_VERSION > 1
 GtkTextMark * logwdw_textmark;
@@ -39,7 +41,6 @@ pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;
 gftp_graphic * gftp_icon;
 pthread_t main_thread_id;
 GList * viewedit_processes = NULL;
-volatile sig_atomic_t viewedit_process_done;
 
 static int
 get_column (GtkCListColumn * col)
@@ -54,9 +55,10 @@ get_column (GtkCListColumn * col)
 
 
 static void
-doexit (GtkWidget * widget, gpointer data)
+_gftp_exit (GtkWidget * widget, gpointer data)
 {
-  int ret;
+  const char *tempstr;
+  intptr_t ret;
 
   ret = GTK_WIDGET (local_frame)->allocation.width;
   gftp_set_global_option ("listbox_local_width", GINT_TO_POINTER (ret));
@@ -98,37 +100,48 @@ doexit (GtkWidget * widget, gpointer data)
   ret = get_column (&GTK_CLIST (window2.listbox)->column[6]);
   gftp_set_global_option ("remote_attribs_width", GINT_TO_POINTER (ret));
 
+  tempstr = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry));
+  gftp_set_global_option ("host_value", tempstr);
+
+  tempstr = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (portedit)->entry));
+  gftp_set_global_option ("port_value", tempstr);
+
+  tempstr = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (useredit)->entry));
+  gftp_set_global_option ("user_value", tempstr);
+
   gftp_shutdown ();
   exit (0);
 }
 
 
 static gint
-delete_event (GtkWidget * widget, GdkEvent * event, gpointer data)
+_gftp_try_close (GtkWidget * widget, GdkEvent * event, gpointer data)
 {
   if (gftp_file_transfers == NULL)
-    doexit (NULL, NULL);
+    {
+      _gftp_exit (NULL, NULL);
+      return (0);
+    }
   else
     {
-      MakeYesNoDialog (_("Exit"), _("There are file transfers in progress.\nAre you sure you want to exit?"), doexit, NULL, NULL, NULL);
-      return (TRUE);
+      MakeYesNoDialog (_("Exit"), _("There are file transfers in progress.\nAre you sure you want to exit?"), _gftp_exit, NULL, NULL, NULL);
+      return (1);
     }
-  return (FALSE);
 }
 
 
 static void
-destroy (GtkWidget * widget, gpointer data)
+_gftp_force_close (GtkWidget * widget, gpointer data)
 {
   exit (0);
 }
 
 
 static void
-menu_exit (GtkWidget * widget, gpointer data)
+_gftp_menu_exit (GtkWidget * widget, gpointer data)
 {
-  if (!delete_event (widget, NULL, data))
-    doexit (widget, data);
+  if (!_gftp_try_close (widget, NULL, data))
+    _gftp_exit (widget, data);
 }
 
 
@@ -137,22 +150,22 @@ change_setting (gftp_window_data * wdata, int menuitem, GtkWidget * checkmenu)
 {
   switch (menuitem)
     {
-    case 1:
+    case GFTP_MENU_ITEM_ASCII:
       gftp_set_global_option ("ascii_transfers", GINT_TO_POINTER(1));
       break;
-    case 2:
+    case GFTP_MENU_ITEM_BINARY:
       gftp_set_global_option ("ascii_transfers", GINT_TO_POINTER(0));
       break;
-    case 3:
+    case GFTP_MENU_ITEM_WIN1:
       current_wdata = &window1;
       other_wdata = &window2;
-      if (wdata->request)
+      if (wdata->request != NULL)
         update_window_info ();
       break;
-    case 4:
+    case GFTP_MENU_ITEM_WIN2:
       current_wdata = &window2;
       other_wdata = &window1;
-      if (wdata->request)
+      if (wdata->request != NULL)
         update_window_info ();
       break;
     }
@@ -196,7 +209,7 @@ tb_openurl_dialog (gpointer data)
 
   if (current_wdata->request->stopable)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
                _("%s: Please hit the stop button first to do anything else\n"),
                _("OpenURL"));
       return;
@@ -213,30 +226,41 @@ tb_openurl_dialog (gpointer data)
 }
 
 
+static void
+gftp_gtk_refresh (gftp_window_data * wdata)
+{
+  gftpui_refresh (wdata, 1);
+}
+
+
 static GtkWidget *
 CreateMenus (GtkWidget * parent)
 {
-  intptr_t local_len, remote_len, len, i, trans_len, log_len, tools_len,
-      ascii_transfers;
+  int local_len, remote_len, len, i, trans_len, log_len, tools_len;
   GtkAccelGroup *accel_group;
+  intptr_t ascii_transfers;
   GtkWidget * tempwid;
   static GtkItemFactoryEntry menu_items[] = {
     {N_("/_FTP"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/FTP/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/FTP/Window 1"), NULL, change_setting, 3, MN_("<RadioItem>")},
-    {N_("/FTP/Window 2"), NULL, change_setting, 4, MN_("/FTP/Window 1")},
+    {N_("/FTP/Window 1"), NULL, change_setting, GFTP_MENU_ITEM_WIN1,
+     MN_("<RadioItem>")},
+    {N_("/FTP/Window 2"), NULL, change_setting, GFTP_MENU_ITEM_WIN2,
+     MN_("/FTP/Window 1")},
     {N_("/FTP/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/FTP/Ascii"), NULL, change_setting, 1, MN_("<RadioItem>")},
-    {N_("/FTP/Binary"), NULL, change_setting, 2, MN_("/FTP/Ascii")},
+    {N_("/FTP/Ascii"), NULL, change_setting, GFTP_MENU_ITEM_ASCII,
+     MN_("<RadioItem>")},
+    {N_("/FTP/Binary"), NULL, change_setting, GFTP_MENU_ITEM_BINARY,
+     MN_("/FTP/Ascii")},
     {N_("/FTP/sep"), NULL, 0, 0, MN_("<Separator>")},
     {N_("/FTP/_Options..."), "<control>O", options_dialog, 0,
 	MS_(GTK_STOCK_PREFERENCES)},
     {N_("/FTP/sep"), NULL, 0, 0, MN_("<Separator>")},
-    {N_("/FTP/_Quit"), "<control>Q", menu_exit, 0, MS_(GTK_STOCK_QUIT)},
+    {N_("/FTP/_Quit"), "<control>Q", _gftp_menu_exit, 0, MS_(GTK_STOCK_QUIT)},
     {N_("/_Local"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Local/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Local/Open _URL..."), NULL, openurl_dialog, 0, MS_(GTK_STOCK_OPEN)},
-    {N_("/Local/Disconnect"), NULL, gftpui_disconnect, 0, MS_(GTK_STOCK_CLOSE)},
+    {N_("/Local/Open URL..."), "", openurl_dialog, 0, MS_(GTK_STOCK_OPEN)},
+    {N_("/Local/Disconnect"), "", gftpui_disconnect, 0, MS_(GTK_STOCK_CLOSE)},
     {N_("/Local/sep"), NULL, 0, 0, MN_("<Separator>")},
     {N_("/Local/Change Filespec..."), NULL, change_filespec, 0, MN_(NULL)},
     {N_("/Local/Show selected"), NULL, show_selected, 0, MN_(NULL)},
@@ -253,7 +277,7 @@ CreateMenus (GtkWidget * parent)
     {N_("/Local/Delete..."), NULL, delete_dialog, 0, MN_(NULL)},
     {N_("/Local/Edit..."), NULL, edit_dialog, 0, MN_(NULL)},
     {N_("/Local/View..."), NULL, view_dialog, 0, MN_(NULL)},
-    {N_("/Local/Refresh"), NULL, gftpui_refresh, 0, MS_(GTK_STOCK_REFRESH)},
+    {N_("/Local/Refresh"), NULL, gftp_gtk_refresh, 0, MS_(GTK_STOCK_REFRESH)},
     {N_("/_Remote"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Remote/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
     {N_("/Remote/Open _URL..."), "<control>U", openurl_dialog, 0,
@@ -276,7 +300,7 @@ CreateMenus (GtkWidget * parent)
     {N_("/Remote/Delete..."), NULL, delete_dialog, 0, MN_(NULL)},
     {N_("/Remote/Edit..."), NULL, edit_dialog, 0, MN_(NULL)},
     {N_("/Remote/View..."), NULL, view_dialog, 0, MN_(NULL)},
-    {N_("/Remote/Refresh"), NULL, gftpui_refresh, 0, MS_(GTK_STOCK_REFRESH)},
+    {N_("/Remote/Refresh"), NULL, gftp_gtk_refresh, 0, MS_(GTK_STOCK_REFRESH)},
     {N_("/_Bookmarks"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Bookmarks/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
     {N_("/Bookmarks/Add bookmark"), "<control>A", add_bookmark, 0,
@@ -292,9 +316,9 @@ CreateMenus (GtkWidget * parent)
     {N_("/Transfers/Skip Current File"), NULL, skip_transfer, 0, MN_(NULL)},
     {N_("/Transfers/Remove File"), NULL, remove_file_transfer, 0,
 	MS_(GTK_STOCK_DELETE)},
-    {N_("/Transfers/Move File _Up"), NULL, move_transfer_up, 0,
+    {N_("/Transfers/Move File Up"), NULL, move_transfer_up, 0,
 	MS_(GTK_STOCK_GO_UP)},
-    {N_("/Transfers/Move File _Down"), NULL, move_transfer_down, 0,
+    {N_("/Transfers/Move File Down"), NULL, move_transfer_down, 0,
 	MS_(GTK_STOCK_GO_DOWN)},
     {N_("/Transfers/sep"), NULL, 0, 0, MN_("<Separator>")},
     {N_("/Transfers/Retrieve Files"), "<control>R", get_files, 0, MN_(NULL)},
@@ -302,15 +326,15 @@ CreateMenus (GtkWidget * parent)
     {N_("/L_ogging"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Logging/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
     {N_("/Logging/Clear"), NULL, clearlog, 0, MS_(GTK_STOCK_CLEAR)},
-    {N_("/Logging/View log..."), NULL, viewlog, 0, MN_(NULL)},
+    {N_("/Logging/View log"), NULL, viewlog, 0, MN_(NULL)},
     {N_("/Logging/Save log..."), NULL, savelog, 0, MS_(GTK_STOCK_SAVE)},
     {N_("/Tool_s"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Tools/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
     {N_("/Tools/Compare Windows"), NULL, compare_windows, 0, MN_(NULL)},
     {N_("/Tools/Clear Cache"), NULL, clear_cache, 0, MS_(GTK_STOCK_CLEAR)},
-    {N_("/_Help"), NULL, 0, 0, MN_("<Branch>")},
+    {N_("/Help"), NULL, 0, 0, MN_("<Branch>")},
     {N_("/Help/tearoff"), NULL, 0, 0, MN_("<Tearoff>")},
-    {N_("/Help/About..."), NULL, about_dialog, 0, MS_(GTK_STOCK_HELP)}
+    {N_("/Help/About"), NULL, about_dialog, 0, MS_(GTK_STOCK_HELP)}
   };
 
   menus = menu_items;
@@ -407,7 +431,7 @@ CreateConnectToolbar (GtkWidget * parent)
   };
   GtkWidget *toolbar, *box, *tempwid;
   gftp_config_list_vars * tmplistvar;
-  char *default_protocol;
+  char *default_protocol, *tempstr;
   int i, num;
 
   toolbar = gtk_handle_box_new ();
@@ -416,6 +440,8 @@ CreateConnectToolbar (GtkWidget * parent)
   gtk_container_add (GTK_CONTAINER (toolbar), box);
   gtk_container_border_width (GTK_CONTAINER (box), 5);
 
+  openurl_tooltip = gtk_tooltips_new ();
+
   tempwid = toolbar_pixmap (parent, "connect.xpm");
   openurl_btn = gtk_button_new ();
   gtk_container_add (GTK_CONTAINER (openurl_btn), tempwid);
@@ -428,7 +454,12 @@ CreateConnectToolbar (GtkWidget * parent)
   gtk_container_border_width (GTK_CONTAINER (openurl_btn), 1);
   gtk_box_pack_start (GTK_BOX (box), openurl_btn, FALSE, FALSE, 0);
 
+#if GTK_MAJOR_VERSION == 1
   tempwid = gtk_label_new (_("Host: "));
+#else
+  tempwid = gtk_label_new_with_mnemonic (_("_Host: "));
+#endif
+
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
 
   hostedit = gtk_combo_new ();
@@ -443,7 +474,13 @@ CreateConnectToolbar (GtkWidget * parent)
     gtk_combo_set_popdown_strings (GTK_COMBO (hostedit), tmplistvar->list);
 
   gtk_combo_disable_activate (GTK_COMBO (hostedit));
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry), "");
+
+  gftp_lookup_global_option ("host_value", &tempstr);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry), tempstr);
+#if GTK_MAJOR_VERSION > 1
+  gtk_label_set_mnemonic_widget (GTK_LABEL (tempwid),
+                                 GTK_COMBO (hostedit)->entry);
+#endif
   gtk_box_pack_start (GTK_BOX (box), hostedit, TRUE, TRUE, 0);
 
   tempwid = gtk_label_new (_("Port: "));
@@ -451,7 +488,7 @@ CreateConnectToolbar (GtkWidget * parent)
 
   portedit = gtk_combo_new ();
   gtk_combo_set_case_sensitive (GTK_COMBO (portedit), 1);
-  gtk_widget_set_size_request (portedit, 45, -1);
+  gtk_widget_set_size_request (portedit, 50, -1);
 
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO (portedit)->entry), "activate",
 		      GTK_SIGNAL_FUNC (toolbar_hostedit), NULL);
@@ -461,10 +498,16 @@ CreateConnectToolbar (GtkWidget * parent)
     gtk_combo_set_popdown_strings (GTK_COMBO (portedit), tmplistvar->list);
 
   gtk_combo_disable_activate (GTK_COMBO (portedit));
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), "");
+
+  gftp_lookup_global_option ("port_value", &tempstr);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), tempstr);
   gtk_box_pack_start (GTK_BOX (box), portedit, FALSE, FALSE, 0);
 
+#if GTK_MAJOR_VERSION == 1
   tempwid = gtk_label_new (_("User: "));
+#else
+  tempwid = gtk_label_new_with_mnemonic (_("_User: "));
+#endif
   gtk_box_pack_start (GTK_BOX (box), tempwid, FALSE, FALSE, 0);
 
   useredit = gtk_combo_new ();
@@ -479,7 +522,13 @@ CreateConnectToolbar (GtkWidget * parent)
     gtk_combo_set_popdown_strings (GTK_COMBO (useredit), tmplistvar->list);
 
   gtk_combo_disable_activate (GTK_COMBO (useredit));
-  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (useredit)->entry), "");
+
+  gftp_lookup_global_option ("user_value", &tempstr);
+  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (useredit)->entry), tempstr);
+#if GTK_MAJOR_VERSION > 1
+  gtk_label_set_mnemonic_widget (GTK_LABEL (tempwid),
+                                 GTK_COMBO (useredit)->entry);
+#endif
   gtk_box_pack_start (GTK_BOX (box), useredit, TRUE, TRUE, 0);
 
   tempwid = gtk_label_new (_("Pass: "));
@@ -527,6 +576,7 @@ CreateConnectToolbar (GtkWidget * parent)
 #endif
 
   stop_btn = gtk_button_new ();
+
   gtk_container_add (GTK_CONTAINER (stop_btn), tempwid);
   gtk_widget_set_sensitive (stop_btn, 0);
   gtk_signal_connect_object (GTK_OBJECT (stop_btn), "clicked",
@@ -580,34 +630,33 @@ list_doaction (gftp_window_data * wdata)
 {
   intptr_t list_dblclk_action;
   GList *templist, *filelist;
-  int num, dir, success;
   gftp_file *tempfle;
+  int num, success;
   char *directory;
 
   gftp_lookup_request_option (wdata->request, "list_dblclk_action", 
                               &list_dblclk_action);
 
   filelist = wdata->files;
-  templist = GTK_CLIST (wdata->listbox)->selection;
+  templist = gftp_gtk_get_list_selection (wdata);
   num = 0;
   templist = get_next_selection (templist, &filelist, &num);
   tempfle = (gftp_file *) filelist->data;
 
-  dir = tempfle->isdir;
-
   if (check_reconnect (wdata) < 0) 
     return;
 
-  if (tempfle->islink || tempfle->isdir)
+  if (S_ISLNK (tempfle->st_mode) || S_ISDIR (tempfle->st_mode))
     {
-      directory = gftp_build_path (wdata->request->directory, tempfle->file, NULL);
+      directory = gftp_build_path (wdata->request, wdata->request->directory,
+                                   tempfle->file, NULL);
       success = gftpui_run_chdir (wdata, directory);
       g_free (directory);
     }
   else
     success = 0;
 
-  if (!dir && !success)
+  if (!S_ISDIR (tempfle->st_mode) && !success)
     {
       switch (list_dblclk_action)
         {
@@ -667,7 +716,7 @@ list_dblclick (GtkWidget * widget, GdkEventButton * event, gpointer data)
 }
 
 
-void 
+static void 
 select_row_callback (GtkWidget *widget, gint row, gint column,
                      GdkEventButton *event, gpointer data)
 {
@@ -800,11 +849,11 @@ CreateFTPWindow (gftp_window_data * wdata)
 static gint
 menu_mouse_click (GtkWidget * widget, GdkEventButton * event, gpointer data)
 {
-  GtkItemFactory *factory;
+  GtkItemFactory *mfactory;
 
-  factory = (GtkItemFactory *) data;
+  mfactory = (GtkItemFactory *) data;
   if (event->button == 3)
-    gtk_item_factory_popup (factory, (guint) event->x_root,
+    gtk_item_factory_popup (mfactory, (guint) event->x_root,
 			    (guint) event->y_root, 3, event->time);
   return (FALSE);
 }
@@ -813,8 +862,7 @@ menu_mouse_click (GtkWidget * widget, GdkEventButton * event, gpointer data)
 static GtkWidget *
 CreateFTPWindows (GtkWidget * ui)
 {
-  GtkWidget *box, *dlbox, *winpane, *dlpane, *logpane, *mainvbox, *tempwid,
-            *button;
+  GtkWidget *box, *dlbox, *winpane, *dlpane, *logpane, *mainvbox, *tempwid;
   gftp_config_list_vars * tmplistvar;
   char *dltitles[2];
   intptr_t tmplookup;
@@ -866,11 +914,11 @@ CreateFTPWindows (GtkWidget * ui)
                                       GTK_ICON_SIZE_SMALL_TOOLBAR);
 #endif
 
-  button = gtk_button_new ();
-  gtk_box_pack_start (GTK_BOX (dlbox), button, TRUE, FALSE, 0);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+  upload_right_arrow = gtk_button_new ();
+  gtk_box_pack_start (GTK_BOX (dlbox), upload_right_arrow, TRUE, FALSE, 0);
+  gtk_signal_connect_object (GTK_OBJECT (upload_right_arrow), "clicked",
 			     GTK_SIGNAL_FUNC (put_files), NULL);
-  gtk_container_add (GTK_CONTAINER (button), tempwid);
+  gtk_container_add (GTK_CONTAINER (upload_right_arrow), tempwid);
 
 #if GTK_MAJOR_VERSION == 1
   tempwid = toolbar_pixmap (ui, "left.xpm");
@@ -879,11 +927,11 @@ CreateFTPWindows (GtkWidget * ui)
                                       GTK_ICON_SIZE_SMALL_TOOLBAR);
 #endif
 
-  button = gtk_button_new ();
-  gtk_box_pack_start (GTK_BOX (dlbox), button, TRUE, FALSE, 0);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+  download_left_arrow = gtk_button_new ();
+  gtk_box_pack_start (GTK_BOX (dlbox), download_left_arrow, TRUE, FALSE, 0);
+  gtk_signal_connect_object (GTK_OBJECT (download_left_arrow), "clicked",
 			     GTK_SIGNAL_FUNC (get_files), NULL);
-  gtk_container_add (GTK_CONTAINER (button), tempwid);
+  gtk_container_add (GTK_CONTAINER (download_left_arrow), tempwid);
 
   gtk_paned_pack1 (GTK_PANED (winpane), box, 1, 1);
 
@@ -987,7 +1035,7 @@ CreateFTPWindows (GtkWidget * ui)
 }
 
 
-void
+static void
 init_gftp (int argc, char *argv[], GtkWidget * parent)
 {
   if (argc == 2 && strncmp (argv[1], "--", 2) != 0)
@@ -1011,14 +1059,14 @@ toolbar_hostedit (GtkWidget * widget, gpointer data)
 
   if (current_wdata->request->stopable)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
                _("%s: Please hit the stop button first to do anything else\n"),
                _("Connect"));
       return;
     }
 
   if (GFTP_IS_CONNECTED (current_wdata->request))
-    gftpui_disconnect (current_wdata);
+    gftp_disconnect (current_wdata->request);
 
   tempwid = gtk_menu_get_active (GTK_MENU (protocol_menu));
   num = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (tempwid)));
@@ -1092,16 +1140,18 @@ void
 sortrows (GtkCList * clist, gint column, gpointer data)
 {
   char sortcol_name[25], sortasds_name[25];
-  int swap_col;
   intptr_t sortcol, sortasds;
   gftp_window_data * wdata;
   GtkWidget * sort_wid;
   GList * templist;
+  int swap_col;
 
   wdata = data;
-  g_snprintf (sortcol_name, sizeof (sortcol_name), "%s_sortcol", wdata->prefix_col_str);
+  g_snprintf (sortcol_name, sizeof (sortcol_name), "%s_sortcol",
+              wdata->prefix_col_str);
   gftp_lookup_global_option (sortcol_name, &sortcol);
-  g_snprintf (sortasds_name, sizeof (sortasds_name), "%s_sortasds", wdata->prefix_col_str);
+  g_snprintf (sortasds_name, sizeof (sortasds_name), "%s_sortasds",
+              wdata->prefix_col_str);
   gftp_lookup_global_option (sortasds_name, &sortasds);
 
   if (column == -1)
@@ -1227,8 +1277,14 @@ main (int argc, char **argv)
   gftp_option_types[gftp_option_type_color].write_function = gftp_gtk_config_file_write_color;
 
   gftpui_common_init (&argc, &argv, ftp_log);
+  gftpui_common_child_process_done = 0;
 
   g_thread_init (NULL);
+
+#if GTK_MAJOR_VERSION > 1
+  gdk_threads_init();
+#endif
+
   main_thread_id = pthread_self ();
   gtk_set_locale ();
   gtk_init (&argc, &argv);
@@ -1238,9 +1294,9 @@ main (int argc, char **argv)
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_signal_connect (GTK_OBJECT (window), "delete_event",
-		      GTK_SIGNAL_FUNC (delete_event), NULL);
+		      GTK_SIGNAL_FUNC (_gftp_try_close), NULL);
   gtk_signal_connect (GTK_OBJECT (window), "destroy",
-		      GTK_SIGNAL_FUNC (destroy), NULL);
+		      GTK_SIGNAL_FUNC (_gftp_force_close), NULL);
   gtk_window_set_title (GTK_WINDOW (window), gftp_version);
   gtk_window_set_wmclass (GTK_WINDOW(window), "main", "gFTP");
   gtk_widget_set_name (window, gftp_version);
@@ -1268,7 +1324,7 @@ main (int argc, char **argv)
     {
       gftp_setup_startup_directory (window1.request);
       gftp_connect (window1.request);
-      ftp_list_files (&window1, 0);
+      ftp_list_files (&window1);
     }
 
   /* On the remote window, even though we aren't connected, draw the sort
