@@ -17,7 +17,7 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-/* $Id: gftp-gtk.h,v 1.32 2004/03/16 12:23:54 masneyb Exp $ */
+/* $Id: gftp-gtk.h,v 1.39 2004/11/02 01:33:26 masneyb Exp $ */
 
 #ifndef __GFTP_GTK_H
 #define __GFTP_GTK_H
@@ -28,8 +28,14 @@
 #include <gdk/gdkkeysyms.h>
 #include <pthread.h>
 
+#define GFTP_MENU_ITEM_ASCII	1
+#define GFTP_MENU_ITEM_BINARY	2
+#define GFTP_MENU_ITEM_WIN1	3
+#define GFTP_MENU_ITEM_WIN2	4
+
 #define IS_ONE_SELECTED(wdata)		(GTK_CLIST ((wdata)->listbox)->selection && GTK_CLIST ((wdata)->listbox)->selection->next == NULL)
 #define IS_NONE_SELECTED(wdata)		(GTK_CLIST ((wdata)->listbox)->selection == NULL)
+#define gftp_gtk_get_list_selection(wdata)  (GTK_CLIST ((wdata)->listbox)->selection)
 
 #define GFTP_IS_SAME_HOST_START_TRANS(wdata,trequest) \
   ((wdata) != NULL && (wdata)->request != NULL && \
@@ -173,7 +179,9 @@ typedef struct gftp_options_dialog_data_tag
 extern gftp_window_data window1, window2, * other_wdata, * current_wdata;
 extern GtkWidget * stop_btn, * hostedit, * useredit, * passedit,
                  * portedit, * logwdw, * dlwdw, * protocol_menu, * optionmenu,
-                 * gftpui_command_widget;
+                 * gftpui_command_widget, * download_left_arrow,
+                 * upload_right_arrow, * openurl_btn;
+extern GtkTooltips * openurl_tooltip;
 extern GtkAdjustment * logwdw_vadj;
 #if GTK_MAJOR_VERSION > 1
 extern GtkTextMark * logwdw_textmark;
@@ -186,7 +194,6 @@ extern pthread_mutex_t log_mutex;
 extern gftp_graphic * gftp_icon;
 extern pthread_t main_thread_id;
 extern GList * viewedit_processes;
-extern volatile sig_atomic_t viewedit_process_done;
 
 
 /* bookmarks.c */
@@ -386,8 +393,7 @@ void gftp_gtk_setup_bookmark_options 		( GtkWidget * notebook,
 void gftp_gtk_save_bookmark_options 		( gftp_bookmarks_var * bm );
 
 /* transfer.c */
-int ftp_list_files				( gftp_window_data * wdata,
-						  int usecache );
+int ftp_list_files				( gftp_window_data * wdata );
 
 int ftp_connect					( gftp_window_data * wdata,
 						  gftp_request * request,
@@ -402,6 +408,9 @@ void put_files 					( gpointer data );
 void transfer_window_files 			( gftp_window_data * fromwdata,
 						  gftp_window_data * towdata );
 
+int gftp_gtk_get_subdirs 			( gftp_transfer * transfer,
+						  pthread_t *tid );
+
 void *do_getdir_thread 				( void * data );
 
 void start_transfer				( gpointer data );
