@@ -17,8 +17,8 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-#include <gftp-gtk.h>
-static const char cvsid[] = "$Id: menu-items.c,v 1.33 2004/03/21 13:27:02 masneyb Exp $";
+#include "gftp-gtk.h"
+static const char cvsid[] = "$Id: menu-items.c,v 1.40 2005/01/25 02:34:19 masneyb Exp $";
 
 static void
 dochange_filespec (gftp_window_data * wdata, gftp_dialog_data * ddata)
@@ -33,7 +33,7 @@ dochange_filespec (gftp_window_data * wdata, gftp_dialog_data * ddata)
   edttext = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
   if (*edttext == '\0')
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
                _("Change Filespec: Operation canceled...you must enter a string\n"));
       return;
     }
@@ -42,7 +42,7 @@ dochange_filespec (gftp_window_data * wdata, gftp_dialog_data * ddata)
   wdata->filespec = g_strdup (edttext);
 
   filelist = wdata->files;
-  templist = GTK_CLIST (wdata->listbox)->selection;
+  templist = gftp_gtk_get_list_selection (wdata);
   num = 0;
   while (templist != NULL)
     {
@@ -101,7 +101,7 @@ dosave_directory_listing (GtkWidget * widget, gftp_save_dir_struct * str)
   filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (str->filew));
   if ((fd = fopen (filename, "w")) == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL, 
+      ftp_log (gftp_logging_error, NULL, 
                _("Error: Cannot open %s for writing: %s\n"), filename, 
                g_strerror (errno));
       return;
@@ -164,7 +164,7 @@ show_selected (gpointer data)
   wdata->show_selected = 1;
 
   filelist = wdata->files;
-  templist = GTK_CLIST (wdata->listbox)->selection;
+  templist = gftp_gtk_get_list_selection (wdata);
   num = 0;
   while (templist != NULL)
     {
@@ -216,7 +216,7 @@ selectallfiles (gpointer data)
       tempfle = (gftp_file *) templist->data;
       if (tempfle->shown)
 	{
-	  if (tempfle->isdir)
+	  if (S_ISDIR (tempfle->st_mode))
 	    gtk_clist_unselect_row (GTK_CLIST (wdata->listbox), i, 0);
 	  else
 	    gtk_clist_select_row (GTK_CLIST (wdata->listbox), i, 0);
@@ -262,7 +262,7 @@ chdir_edit (GtkWidget * widget, gpointer data)
       return (0);
     }
 
-  if ((tempstr = expand_path (edttxt)) == NULL)
+  if ((tempstr = gftp_expand_path (wdata->request, edttxt)) == NULL)
     return (FALSE);
 
   if (gftpui_run_chdir (wdata, tempstr))
@@ -309,7 +309,7 @@ viewlog (gpointer data)
   tempstr = g_strconcat (g_get_tmp_dir (), "/gftp-view.XXXXXXXXXX", NULL);
   if ((fd = mkstemp (tempstr)) < 0)
     {
-      ftp_log (gftp_logging_misc, NULL, 
+      ftp_log (gftp_logging_error, NULL, 
                _("Error: Cannot open %s for writing: %s\n"), tempstr, 
                g_strerror (errno));
       g_free (tempstr); 
@@ -326,6 +326,10 @@ viewlog (gpointer data)
   gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
   gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, textlen);
   txt = gtk_text_buffer_get_text (textbuf, &iter, &iter2, 0);
+
+  /* gtk_text_buffer_get_char_count() returns the number of characters,
+     not bytes. So get the number of bytes that need to be written out */
+  textlen = strlen (txt);
 #endif
   pos = txt;
 
@@ -333,7 +337,7 @@ viewlog (gpointer data)
     {
       if ((len = write (fd, pos, textlen)) == -1)
         { 
-          ftp_log (gftp_logging_misc, NULL, 
+          ftp_log (gftp_logging_error, NULL, 
                    _("Error: Error writing to %s: %s\n"), 
                    tempstr, g_strerror (errno));
           break;
@@ -369,7 +373,7 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
   filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
   if ((fd = fopen (filename, "w")) == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL, 
+      ftp_log (gftp_logging_error, NULL, 
                _("Error: Cannot open %s for writing: %s\n"), filename, 
                g_strerror (errno));
       return;
@@ -384,6 +388,10 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
   gtk_text_buffer_get_iter_at_offset (textbuf, &iter, 0);
   gtk_text_buffer_get_iter_at_offset (textbuf, &iter2, textlen);
   txt = gtk_text_buffer_get_text (textbuf, &iter, &iter2, 0);
+
+  /* gtk_text_buffer_get_char_count() returns the number of characters,
+     not bytes. So get the number of bytes that need to be written out */
+  textlen = strlen (txt);
 #endif
 
   ok = 1;
@@ -393,7 +401,7 @@ dosavelog (GtkWidget * widget, GtkFileSelection * fs)
       if ((len = write (fileno (fd), pos, textlen)) == -1)
         {
           ok = 0;
-          ftp_log (gftp_logging_misc, NULL, 
+          ftp_log (gftp_logging_error, NULL, 
                    _("Error: Error writing to %s: %s\n"), 
                    filename, g_strerror (errno));
           break;
@@ -442,9 +450,8 @@ clear_cache (gpointer data)
 void 
 about_dialog (gpointer data)
 {
-  GtkWidget * tempwid, * notebook, * box, * label, * view, * vscroll,
-            * dialog;
-  char *tempstr, *no_license_agreement, *str, buf[255];
+  GtkWidget * tempwid, * notebook, * box, * label, * view, * vscroll, * dialog;
+  char *tempstr, *temp1str, *no_license_agreement, *str, buf[255], *share_dir;
   size_t len;
   FILE * fd;
 #if GTK_MAJOR_VERSION > 1
@@ -453,7 +460,8 @@ about_dialog (gpointer data)
   guint textlen;
 #endif
 
-  no_license_agreement = g_strdup_printf (_("Cannot find the license agreement file COPYING. Please make sure it is in either %s or in %s"), BASE_CONF_DIR, SHARE_DIR);
+  share_dir = gftp_get_share_dir ();
+  no_license_agreement = g_strdup_printf (_("Cannot find the license agreement file COPYING. Please make sure it is in either %s or in %s"), BASE_CONF_DIR, share_dir);
 
 #if GTK_MAJOR_VERSION == 1
   dialog = gtk_dialog_new ();
@@ -579,11 +587,13 @@ about_dialog (gpointer data)
   if (access (tempstr, F_OK) != 0)
     {
       g_free (tempstr);
-      tempstr = expand_path (SHARE_DIR "/COPYING");
+      temp1str = g_strconcat (share_dir, "/COPYING", NULL);
+      tempstr = gftp_expand_path (NULL, temp1str);
+      g_free (temp1str);
       if (access (tempstr, F_OK) != 0)
 	{
 	  g_free (tempstr);
-          tempstr = expand_path (BASE_CONF_DIR "/COPYING");
+          tempstr = gftp_expand_path (NULL, BASE_CONF_DIR "/COPYING");
 	  if (access (tempstr, F_OK) != 0)
 	    {
 #if GTK_MAJOR_VERSION == 1
@@ -640,7 +650,7 @@ compare_windows (gpointer data)
 {
   gftp_file * curfle, * otherfle;
   GList * curlist, * otherlist;
-  int row;
+  int row, curdir, othdir;
 
   if (!check_status (_("Compare Windows"), &window2, 1, 0, 0, 1))
     return;
@@ -669,9 +679,12 @@ compare_windows (gpointer data)
               continue;
             }
 
+          curdir = S_ISDIR (curfle->st_mode);
+          othdir = S_ISDIR (otherfle->st_mode);
+
           if (strcmp (otherfle->file, curfle->file) == 0 &&
-              otherfle->isdir == curfle->isdir &&
-              (curfle->isdir || otherfle->size == curfle->size))
+              curdir == othdir &&
+              (curdir || otherfle->size == curfle->size))
 	    break;
 
           otherlist = otherlist->next;
@@ -704,9 +717,12 @@ compare_windows (gpointer data)
               continue;
             }
 
+          curdir = S_ISDIR (curfle->st_mode);
+          othdir = S_ISDIR (otherfle->st_mode);
+
           if (strcmp (otherfle->file, curfle->file) == 0 &&
-              otherfle->isdir == curfle->isdir &&
-              (curfle->isdir || otherfle->size == curfle->size))
+              curdir == othdir &&
+              (curdir || otherfle->size == curfle->size))
 	    break;
 
           otherlist = otherlist->next;
