@@ -17,8 +17,8 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-#include <gftp-gtk.h>
-static const char cvsid[] = "$Id: misc-gtk.c,v 1.32 2003/08/12 01:05:02 masneyb Exp $";
+#include "gftp-gtk.h"
+static const char cvsid[] = "$Id: misc-gtk.c,v 1.56 2005/01/25 02:34:19 masneyb Exp $";
 
 static GtkWidget * statuswid;
 
@@ -39,21 +39,21 @@ void
 ftp_log (gftp_logging_level level, gftp_request * request, 
          const char *string, ...)
 {
-  guint max_log_window_size;
+  uintptr_t max_log_window_size;
   int upd, free_logstr;
   gftp_log * newlog;
-  char *logstr;
   gint delsize;
+  char *logstr;
   va_list argp;
-  guint len;
+  size_t len;
 #if GTK_MAJOR_VERSION == 1
   gftp_color * color;
   GdkColor fore;
 #else
-  char *utf8_str;
   GtkTextBuffer * textbuf;
   GtkTextIter iter, iter2;
   const char *descr;
+  char *utf8_str;
 #endif
 
   va_start (argp, string);
@@ -70,17 +70,14 @@ ftp_log (gftp_logging_level level, gftp_request * request,
   va_end (argp);
 
 #if GTK_MAJOR_VERSION > 1
-  if (!g_utf8_validate (logstr, -1, NULL))
+  if ((utf8_str = gftp_string_to_utf8 (request, logstr)) != NULL)
     {
-      if ((utf8_str = gftp_string_to_utf8 (request, logstr)) != NULL)
-        {
-          if (free_logstr)
-            g_free (logstr);
-          else
-            free_logstr = 1;
+      if (free_logstr)
+        g_free (logstr);
+      else
+        free_logstr = 1;
 
-          logstr = utf8_str;
-        }
+      logstr = utf8_str;
     }
 #endif
 
@@ -99,7 +96,7 @@ ftp_log (gftp_logging_level level, gftp_request * request,
       return;
     }
 
-  if (gftp_logfd != NULL)
+  if (gftp_logfd != NULL && level != gftp_logging_misc_nolog)
     {
       if (fwrite (logstr, strlen (logstr), 1, gftp_logfd) != 1)
         {
@@ -208,55 +205,56 @@ ftp_log (gftp_logging_level level, gftp_request * request,
 }
 
 
-void
-refresh (gftp_window_data * wdata)
-{
-  if (!check_status (_("Refresh"), wdata, 0, 0, 0, 1))
-    return;
-  gtk_clist_freeze (GTK_CLIST (wdata->listbox));
-  remove_files_window (wdata);
-  gftp_delete_cache_entry (wdata->request, NULL, 0);
-  ftp_list_files (wdata, 0);
-  gtk_clist_thaw (GTK_CLIST (wdata->listbox));
-}
-
-
 void
 update_window_info (void)
 {
   char *tempstr, empty[] = "";
   GtkWidget * tempwid;
+  unsigned int port;
   int i;
 
   if (current_wdata->request != NULL)
     {
-      if ((tempstr = current_wdata->request->hostname) == NULL)
-        tempstr = empty;
-      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry), tempstr);
-
-      if ((tempstr = current_wdata->request->username) == NULL)
-        tempstr = empty;
-      gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (useredit)->entry), tempstr);
-
-      if ((tempstr = current_wdata->request->password) == NULL)
-        tempstr = empty;
-      gtk_entry_set_text (GTK_ENTRY (passedit), tempstr);
-
-      if (current_wdata->request->port != 0)
+      if (GFTP_IS_CONNECTED (current_wdata->request))
         {
-          tempstr = g_strdup_printf ("%d", current_wdata->request->port);
-          gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), tempstr);
-          g_free (tempstr);
-        }
+          if ((tempstr = current_wdata->request->hostname) == NULL)
+            tempstr = empty;
+          gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (hostedit)->entry), tempstr);
+    
+          if ((tempstr = current_wdata->request->username) == NULL)
+            tempstr = empty;
+          gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (useredit)->entry), tempstr);
+    
+          if ((tempstr = current_wdata->request->password) == NULL)
+            tempstr = empty;
+          gtk_entry_set_text (GTK_ENTRY (passedit), tempstr);
+    
+          port = gftp_protocol_default_port (current_wdata->request);
+          if (current_wdata->request->port != 0 &&
+              port != current_wdata->request->port)
+            {
+              tempstr = g_strdup_printf ("%d", current_wdata->request->port);
+              gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), tempstr);
+              g_free (tempstr);
+            }
+          else
+            gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (portedit)->entry), "");
+    
+          for (i=0; gftp_protocols[i].init != NULL; i++)
+            {
+              if (current_wdata->request->init == gftp_protocols[i].init)
+                {
+                  gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), i);
+                  break;
+                }
+            }
 
-     for (i=0; gftp_protocols[i].init != NULL; i++)
-       {
-         if (current_wdata->request->init == gftp_protocols[i].init)
-           {
-             gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu), i);
-             break;
-           }
-       }
+          gtk_tooltips_set_tip (GTK_TOOLTIPS(openurl_tooltip), openurl_btn,
+                                _("Disconnect from the remote server"), NULL);
+        }
+      else
+        gtk_tooltips_set_tip (GTK_TOOLTIPS(openurl_tooltip), openurl_btn,
+                              _("Connect to the site specified in the host entry. If the host entry is blank, then a dialog is presented that will allow you to enter a URL."), NULL);
     }
 
   update_window (&window1);
@@ -281,20 +279,23 @@ set_menu_sensitive (gftp_window_data * wdata, char *path, int sensitive)
   if (tempwid)
     gtk_widget_set_sensitive (tempwid, sensitive);
 
-  if ((pos = strchr (path + 1, '/')) == NULL)
-    pos = path;
-
-  if (wdata->ifactory)
-    tempwid = gtk_item_factory_get_widget (wdata->ifactory, pos);
-  if (tempwid)
-    gtk_widget_set_sensitive (tempwid, sensitive);
+  if (wdata != NULL)
+    {
+      if ((pos = strchr (path + 1, '/')) == NULL)
+        pos = path;
+    
+      if (wdata->ifactory)
+        tempwid = gtk_item_factory_get_widget (wdata->ifactory, pos);
+      if (tempwid)
+        gtk_widget_set_sensitive (tempwid, sensitive);
+    }
 }
 
 
 void
 update_window (gftp_window_data * wdata)
 {
-  char *dir, *tempstr, *temp1str, *fspec;
+  char *dir, *tempstr, *temp1str, *fspec, *utf8_directory;
   int connected, start;
 
   connected = GFTP_IS_CONNECTED (wdata->request);
@@ -312,12 +313,23 @@ update_window (gftp_window_data * wdata)
       gtk_label_set (GTK_LABEL (wdata->hoststxt), tempstr);
       g_free (tempstr);
 
+      utf8_directory = NULL;
       if ((dir = wdata->request->directory) == NULL)
         temp1str = "";
       else
-        temp1str = dir;
+        {
+          utf8_directory = gftp_string_to_utf8 (wdata->request, 
+                                                wdata->request->directory);
+          if (utf8_directory != NULL)
+            temp1str = utf8_directory;
+          else
+            temp1str = dir;
+        }
 
       gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (wdata->combo)->entry),temp1str);
+
+      if (utf8_directory != NULL)
+        g_free (utf8_directory);
     }
   else if (wdata->hoststxt != NULL)
     {
@@ -359,6 +371,23 @@ update_window (gftp_window_data * wdata)
   set_menu_sensitive (wdata, menus[start + 19].path, connected &&
                       wdata->request->get_file != NULL);
   set_menu_sensitive (wdata, menus[start + 20].path, connected);
+
+  connected = GFTP_IS_CONNECTED (window1.request) && GFTP_IS_CONNECTED (window2.request);
+
+  start = trans_start;
+  set_menu_sensitive (NULL, menus[start + 2].path, connected);
+  set_menu_sensitive (NULL, menus[start + 3].path, connected);
+  set_menu_sensitive (NULL, menus[start + 5].path, connected);
+  set_menu_sensitive (NULL, menus[start + 6].path, connected);
+
+  set_menu_sensitive (NULL, menus[start + 7].path, connected);
+  set_menu_sensitive (NULL, menus[start + 8].path, connected);
+
+  set_menu_sensitive (NULL, menus[start + 10].path, connected);
+  set_menu_sensitive (NULL, menus[start + 11].path, connected);
+
+  gtk_widget_set_sensitive (download_left_arrow, connected);
+  gtk_widget_set_sensitive (upload_right_arrow, connected);
 }  
 
 
@@ -479,7 +508,7 @@ check_status (char *name, gftp_window_data *wdata, int check_other_stop,
 
   if (wdata->request->stopable)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("%s: Please hit the stop button first to do anything else\n"),
 	       name);
       return (0);
@@ -487,7 +516,7 @@ check_status (char *name, gftp_window_data *wdata, int check_other_stop,
 
   if (check_other_stop && owdata->request->stopable)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("%s: Please hit the stop button first to do anything else\n"),
 	       name);
       return (0);
@@ -495,14 +524,14 @@ check_status (char *name, gftp_window_data *wdata, int check_other_stop,
 
   if (!GFTP_IS_CONNECTED (wdata->request))
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("%s: Not connected to a remote site\n"), name);
       return (0);
     }
 
   if (!func)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("%s: This feature is not available using this protocol\n"),
 	       name);
       return (0);
@@ -510,14 +539,14 @@ check_status (char *name, gftp_window_data *wdata, int check_other_stop,
 
   if (only_one && !IS_ONE_SELECTED (wdata))
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("%s: You must only have one item selected\n"), name);
       return (0);
     }
 
   if (at_least_one && !only_one && IS_NONE_SELECTED (wdata))
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("%s: You must have at least one item selected\n"), name);
       return (0);
     }
@@ -568,12 +597,15 @@ void
 create_item_factory (GtkItemFactory * ifactory, guint n_entries,
 		     GtkItemFactoryEntry * entries, gpointer callback_data)
 {
-  int i;
-  const char *strip_prefix = gtk_object_get_data (GTK_OBJECT (ifactory), "gftp-strip-prefix");
-  int strip_prefix_len = 0;
+  const char *strip_prefix;
+  size_t strip_prefix_len;
+  size_t i;
 
+  strip_prefix = gtk_object_get_data (GTK_OBJECT (ifactory), "gftp-strip-prefix");
   if (strip_prefix)
     strip_prefix_len = strlen (strip_prefix);
+  else
+    strip_prefix_len = 0;
 
   for (i = 0; i < n_entries; i++)
     {
@@ -651,14 +683,15 @@ check_reconnect (gftp_window_data *wdata)
 void
 add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
 {
-  char *add_data[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL }, *pos;
+  char *add_data[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+  char *tempstr, *str, *pos, *attribs;
   gftp_config_list_vars * tmplistvar;
-  int clist_num, show_hidden_files;
   gftp_file_extensions * tempext;
-  char *tempstr, *str;
+  intptr_t show_hidden_files;
   GdkBitmap * bitmap;
   GList * templist;
   GdkPixmap * pix;
+  int clist_num;
   size_t stlen;
 
   gftp_lookup_request_option (wdata->request, "show_hidden_files", 
@@ -693,13 +726,15 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
   bitmap = NULL;
   if (strcmp (fle->file, "..") == 0)
     gftp_get_pixmap (wdata->listbox, "dotdot.xpm", &pix, &bitmap);
-  else if (fle->islink && fle->isdir)
+  else if (S_ISLNK (fle->st_mode) && S_ISDIR (fle->st_mode))
     gftp_get_pixmap (wdata->listbox, "linkdir.xpm", &pix, &bitmap);
-  else if (fle->islink)
+  else if (S_ISLNK (fle->st_mode))
     gftp_get_pixmap (wdata->listbox, "linkfile.xpm", &pix, &bitmap);
-  else if (fle->isdir)
+  else if (S_ISDIR (fle->st_mode))
     gftp_get_pixmap (wdata->listbox, "dir.xpm", &pix, &bitmap);
-  else if (fle->isexe)
+  else if ((fle->st_mode & S_IXUSR) ||
+           (fle->st_mode & S_IXGRP) ||
+           (fle->st_mode & S_IXOTH))
     gftp_get_pixmap (wdata->listbox, "exe.xpm", &pix, &bitmap); 
   else
     {
@@ -731,7 +766,7 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
   else if (fle->file)
     gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 1, fle->file);
 
-  if (fle->attribs && (*fle->attribs == 'b' || *fle->attribs == 'c'))
+  if (GFTP_IS_SPECIAL_DEVICE (fle->st_mode))
     tempstr = g_strdup_printf ("%d, %d", major (fle->size),
                                minor (fle->size));
   else
@@ -750,9 +785,22 @@ add_file_listbox (gftp_window_data * wdata, gftp_file * fle)
         *pos = '\0';
       gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 5, str);
     }
-  if (fle->attribs)
-    gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 6, fle->attribs);
 
+  attribs = gftp_convert_attributes_from_mode_t (fle->st_mode);
+  gtk_clist_set_text (GTK_CLIST (wdata->listbox), clist_num, 6, attribs);
+  g_free (attribs);
+
+}
+
+
+void
+destroy_dialog (gftp_dialog_data * ddata)
+{
+  if (ddata->dialog != NULL)
+    {
+      gtk_widget_destroy (ddata->dialog);
+      ddata->dialog = NULL;
+    }
 }
 
 
@@ -770,7 +818,8 @@ ok_dialog_response (GtkWidget * widget, gftp_dialog_data * ddata)
   if (ddata->yesfunc != NULL)
     ddata->yesfunc (ddata->yespointer, ddata);
 
-  if (ddata->edit != NULL)
+  if (ddata->edit != NULL &&
+      ddata->dialog != NULL)
     gtk_widget_destroy (ddata->dialog);
 
   g_free (ddata);
@@ -790,7 +839,8 @@ cancel_dialog_response (GtkWidget * widget, gftp_dialog_data * ddata)
   if (ddata->nofunc != NULL)
     ddata->nofunc (ddata->nopointer, ddata);
 
-  if (ddata->edit != NULL)
+  if (ddata->edit != NULL &&
+      ddata->dialog != NULL)
     gtk_widget_destroy (ddata->dialog);
 
   g_free (ddata);
@@ -818,7 +868,8 @@ dialog_response (GtkWidget * widget, gint response, gftp_dialog_data * ddata)
         break;
     }
 
-  if (ddata->edit != NULL)
+  if (ddata->edit != NULL &&
+      ddata->dialog != NULL)
     gtk_widget_destroy (ddata->dialog);
 
   g_free (ddata);
@@ -865,7 +916,7 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
   gftp_dialog_data * ddata;
   const gchar * yes_text;
 
-  ddata = g_malloc (sizeof (*ddata));
+  ddata = g_malloc0 (sizeof (*ddata));
   ddata->yesfunc = okfunc;
   ddata->yespointer = okptr;
   ddata->nofunc = cancelfunc;
@@ -882,6 +933,9 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
 #else
   switch (okbutton)
     {
+      case gftp_dialog_button_ok:
+        yes_text = GTK_STOCK_OK;
+        break;
       case gftp_dialog_button_create:
         yes_text = GTK_STOCK_ADD;
         break;
@@ -953,6 +1007,9 @@ MakeEditDialog (char *diagtxt, char *infotxt, char *deftext, int passwd_item,
 #if GTK_MAJOR_VERSION == 1
   switch (okbutton)
     {
+      case gftp_dialog_button_ok:
+        yes_text = _("OK");
+        break;
       case gftp_dialog_button_create:
         yes_text = _("Add");
         break;
@@ -1008,7 +1065,7 @@ MakeYesNoDialog (char *diagtxt, char *infotxt,
   GtkWidget * tempwid;
 #endif
 
-  ddata = g_malloc (sizeof (*ddata));
+  ddata = g_malloc0 (sizeof (*ddata));
   ddata->yesfunc = yesfunc;
   ddata->yespointer = yespointer;
   ddata->nofunc = nofunc;
@@ -1111,7 +1168,11 @@ update_directory_download_progress (gftp_transfer * transfer)
 
   if (dialog == NULL)
     {
-      dialog = gtk_window_new (GTK_WINDOW_POPUP);
+      dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+      gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
+#if GTK_MAJOR_VERSION > 1
+      gtk_window_set_decorated (GTK_WINDOW (dialog), 0);
+#endif
       gtk_grab_add (dialog);
       gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
                           GTK_SIGNAL_FUNC (delete_event), NULL);
@@ -1152,40 +1213,6 @@ update_directory_download_progress (gftp_transfer * transfer)
 }
 
 
-void *
-generic_thread (void * (*func) (void *), gftp_window_data * wdata)
-{
-  void * ret;
-
-  if (wdata->request->use_threads)
-    {
-      wdata->request->stopable = 1;
-      gtk_widget_set_sensitive (stop_btn, 1);
-      pthread_create (&wdata->tid, NULL, func, wdata);
-
-      while (wdata->request->stopable)
-        {
-          GDK_THREADS_LEAVE ();
-#if GTK_MAJOR_VERSION == 1
-          g_main_iteration (TRUE);
-#else
-          g_main_context_iteration (NULL, TRUE);
-#endif
-        }
-
-      pthread_join (wdata->tid, &ret);
-      gtk_widget_set_sensitive (stop_btn, 0);
-    }
-  else
-    ret = func (wdata);
-
-  if (!GFTP_IS_CONNECTED (wdata->request))
-    disconnect (wdata);
-
-  return (ret); 
-}
-
-
 int
 progress_timeout (gpointer data)
 {
@@ -1198,7 +1225,7 @@ progress_timeout (gpointer data)
 
   val = gtk_progress_get_value (GTK_PROGRESS (statuswid));
   if (val >= 1.0)
-    val = 0;
+    val = 0.0;
   else
     val += 0.10;
   gtk_progress_bar_update (GTK_PROGRESS_BAR (statuswid), val);
@@ -1230,31 +1257,21 @@ display_cached_logs (void)
 }
 
 
-RETSIGTYPE 
-signal_handler (int signo)
-{
-  signal (signo, signal_handler);
-
-  if (use_jmp_environment)
-    siglongjmp (jmp_environment, signo == SIGINT ? 1 : 2);
-  else if (signo == SIGINT)
-    exit (1);
-}
-
-
 char *
 get_xpm_path (char *filename, int quit_on_err)
 {
-  char *tempstr, *exfile;
+  char *tempstr, *exfile, *share_dir;
 
   tempstr = g_strconcat (BASE_CONF_DIR, "/", filename, NULL);
-  exfile = expand_path (tempstr);
+  exfile = gftp_expand_path (NULL, tempstr);
   g_free (tempstr);
   if (access (exfile, F_OK) != 0)
     {
       g_free (exfile);
-      tempstr = g_strconcat (SHARE_DIR, "/", filename, NULL);
-      exfile = expand_path (tempstr);
+      share_dir = gftp_get_share_dir ();
+
+      tempstr = g_strconcat (share_dir, "/", filename, NULL);
+      exfile = gftp_expand_path (NULL, tempstr);
       g_free (tempstr);
       if (access (exfile, F_OK) != 0)
 	{
@@ -1267,7 +1284,7 @@ get_xpm_path (char *filename, int quit_on_err)
 		return (NULL);
 
 	      printf (_("gFTP Error: Cannot find file %s in %s or %s\n"),
-		      filename, SHARE_DIR, BASE_CONF_DIR);
+		      filename, share_dir, BASE_CONF_DIR);
 	      exit (1);
 	    }
 	}
