@@ -17,13 +17,13 @@
 /*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                */
 /*****************************************************************************/
 
-#include <gftp-gtk.h>
-static const char cvsid[] = "$Id: transfer.c,v 1.58 2004/03/17 20:42:53 masneyb Exp $";
+#include "gftp-gtk.h"
+static const char cvsid[] = "$Id: transfer.c,v 1.76 2005/01/25 02:34:19 masneyb Exp $";
 
 static int num_transfers_in_progress = 0;
 
 int
-ftp_list_files (gftp_window_data * wdata, int usecache)
+ftp_list_files (gftp_window_data * wdata)
 {
   gftpui_callback_data * cdata;
 
@@ -33,6 +33,7 @@ ftp_list_files (gftp_window_data * wdata, int usecache)
   cdata->request = wdata->request;
   cdata->uidata = wdata;
   cdata->run_function = gftpui_common_run_ls;
+  cdata->dont_refresh = 1;
 
   gftpui_common_run_callback_function (cdata);
 
@@ -85,9 +86,7 @@ transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
   gftp_file * tempfle, * newfle;
   GList * templist, * filelist;
   gftp_transfer * transfer;
-  guint timeout_num;
-  void *ret;
-  int num;
+  int num, ret, disconnect;
 
   if (!check_status (_("Transfer Files"), fromwdata, 1, 0, 1,
        towdata->request->put_file != NULL && fromwdata->request->get_file != NULL))
@@ -96,7 +95,7 @@ transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
   if (!GFTP_IS_CONNECTED (fromwdata->request) || 
       !GFTP_IS_CONNECTED (towdata->request))
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
                _("Retrieve Files: Not connected to a remote site\n"));
       return;
     }
@@ -111,7 +110,7 @@ transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
   transfer->towdata = towdata;
 
   num = 0;
-  templist = GTK_CLIST (fromwdata->listbox)->selection;
+  templist = gftp_gtk_get_list_selection (fromwdata);
   filelist = fromwdata->files;
   while (templist != NULL)
     {
@@ -129,45 +128,30 @@ transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
       gftp_swap_socks (transfer->fromreq, fromwdata->request);
       gftp_swap_socks (transfer->toreq, towdata->request);
 
-      if (gftpui_common_use_threads (transfer->fromreq) || 
-          (transfer->toreq && gftpui_common_use_threads (transfer->toreq)))
-        {
-          transfer->fromreq->stopable = 1;
-          pthread_create (&fromwdata->tid, NULL, do_getdir_thread, transfer);
-
-          timeout_num = gtk_timeout_add (100, progress_timeout, transfer);
-
-          while (transfer->fromreq->stopable)
-            {
-              GDK_THREADS_LEAVE ();
-#if GTK_MAJOR_VERSION == 1
-              g_main_iteration (TRUE);
-#else
-              g_main_context_iteration (NULL, TRUE);
-#endif
-            }
-
-          gtk_timeout_remove (timeout_num);
-          transfer->numfiles = transfer->numdirs = -1; 
-          update_directory_download_progress (transfer);
-
-          pthread_join (fromwdata->tid, &ret);
-        }
+      ret = gftp_gtk_get_subdirs (transfer, &fromwdata->tid);
+      if (ret < 0)
+        disconnect = 1;
       else
-        ret = do_getdir_thread (transfer);
+        disconnect = 0;
 
       if (!GFTP_IS_CONNECTED (transfer->fromreq))
         {
           gftpui_disconnect (fromwdata);
-          return;
+          disconnect = 1;
         } 
 
       if (!GFTP_IS_CONNECTED (transfer->toreq))
         {
           gftpui_disconnect (towdata);
-          return;
+          disconnect = 1;
         } 
 
+      if (disconnect)
+        {
+          free_tdata (transfer);
+          return;
+        }
+
       gftp_swap_socks (fromwdata->request, transfer->fromreq);
       gftp_swap_socks (towdata->request, transfer->toreq);
     }
@@ -183,42 +167,89 @@ transfer_window_files (gftp_window_data * fromwdata, gftp_window_data * towdata)
     free_tdata (transfer);
 }
 
-void *
-do_getdir_thread (void * data)
+
+static int
+gftpui_gtk_tdata_connect (gftpui_callback_data * cdata)
 {
-  gftp_transfer * transfer;
-  int success, sj;
+  gftp_transfer * tdata;
+  int ret;
 
-  transfer = data;
+  tdata = cdata->user_data;
 
-  if (gftpui_common_use_threads (transfer->fromreq) || 
-      (transfer->toreq && gftpui_common_use_threads (transfer->toreq)))
+  if (tdata->fromreq != NULL)
     {
-      sj = sigsetjmp (gftpui_common_jmp_environment, 1);
-      gftpui_common_use_jmp_environment = 1;
-    }
-  else
-    sj = 0;
-
-  success = 0;
-  if (sj == 0)
-    success = gftp_get_all_subdirs (transfer, NULL) == 0;
-  else
-    {
-      gftp_disconnect (transfer->fromreq);
-      if (transfer->toreq)
-        gftp_disconnect (transfer->toreq);
-      transfer->fromreq->logging_function (gftp_logging_error,
-                                           transfer->fromreq,
-                                           _("Operation canceled\n"));
+      ret = gftp_connect (tdata->fromreq);
+      if (ret < 0)
+        return (ret);
     }
 
-  if (gftpui_common_use_threads (transfer->fromreq) || 
-      (transfer->toreq && gftpui_common_use_threads (transfer->toreq)))
-    gftpui_common_use_jmp_environment = 0;
+  if (tdata->toreq != NULL)
+    {
+      ret = gftp_connect (tdata->toreq);
+      if (ret < 0)
+        return (ret);
+    }
 
-  transfer->fromreq->stopable = 0;
-  return (GINT_TO_POINTER (success));
+  return (0);
+}
+
+
+static void
+gftpui_gtk_tdata_disconnect (gftpui_callback_data * cdata)
+{
+  gftp_transfer * tdata;
+
+  tdata = cdata->user_data;
+
+  if (tdata->fromreq != NULL)
+    gftp_disconnect (tdata->fromreq);
+
+  if (tdata->toreq != NULL)
+    gftp_disconnect (tdata->toreq);
+
+  cdata->request->datafd = -1;
+}
+
+
+static int
+_gftp_getdir_thread (gftpui_callback_data * cdata)
+{
+  return (gftp_get_all_subdirs (cdata->user_data, NULL));
+}
+
+
+int
+gftp_gtk_get_subdirs (gftp_transfer * transfer, pthread_t *tid)
+{
+  gftpui_callback_data * cdata; 
+  long numfiles, numdirs;
+  guint timeout_num;
+  int ret;
+
+  cdata = g_malloc0 (sizeof (*cdata));
+  cdata->user_data = transfer;
+  cdata->uidata = transfer->fromwdata;
+  cdata->request = ((gftp_window_data *) transfer->fromwdata)->request;
+  cdata->run_function = _gftp_getdir_thread;
+  cdata->connect_function = gftpui_gtk_tdata_connect;
+  cdata->disconnect_function = gftpui_gtk_tdata_disconnect;
+  cdata->dont_check_connection = 1;
+  cdata->dont_refresh = 1;
+
+  timeout_num = gtk_timeout_add (100, progress_timeout, transfer);
+  ret = gftpui_common_run_callback_function (cdata);
+  gtk_timeout_remove (timeout_num);
+
+  numfiles = transfer->numfiles;
+  numdirs = transfer->numdirs;
+  transfer->numfiles = transfer->numdirs = -1; 
+  update_directory_download_progress (transfer);
+  transfer->numfiles = numfiles;
+  transfer->numdirs = numdirs;
+
+  g_free (cdata);
+
+  return (ret);
 }
 
 
@@ -269,7 +300,8 @@ do_upload (gftp_viewedit_data * ve_proc, gftp_dialog_data * ddata)
   GList * newfile;
 
   tempfle = g_malloc0 (sizeof (*tempfle));
-  tempfle->destfile = gftp_build_path (ve_proc->torequest->directory,
+  tempfle->destfile = gftp_build_path (ve_proc->torequest,
+                                       ve_proc->torequest->directory,
                                        ve_proc->remote_filename, NULL);
   ve_proc->remote_filename = NULL;
   tempfle->file = ve_proc->filename;
@@ -292,12 +324,12 @@ check_done_process (void)
 {
   gftp_viewedit_data * ve_proc;
   GList * curdata, *deldata;
+  int ret, procret;
   struct stat st;
-  int ret;
   char *str;
   pid_t pid;
 
-  viewedit_process_done = 0;
+  gftpui_common_child_process_done = 0;
   while ((pid = waitpid (-1, &ret, WNOHANG)) > 0)
     {
       curdata = viewedit_processes;
@@ -310,17 +342,31 @@ check_done_process (void)
 	    {
 	      viewedit_processes = g_list_remove_link (viewedit_processes, 
                                                        deldata);
-	      if (ret != 0)
-		ftp_log (gftp_logging_error, NULL,
-			 _("Error: Child %d returned %d\n"), pid, ret);
-	      else
-		ftp_log (gftp_logging_misc, NULL,
-			 _("Child %d returned successfully\n"), pid);
+              if (WIFEXITED (ret))
+                {
+                  procret = WEXITSTATUS (ret);
+                  if (procret != 0)
+                    {
+                      ftp_log (gftp_logging_error, NULL,
+                               _("Error: Child %d returned %d\n"), pid, procret);
+                      remove_file (ve_proc->filename);
+                      continue;
+                    }
+                  else
+                    ftp_log (gftp_logging_misc, NULL,
+                             _("Child %d returned successfully\n"), pid);
+                }
+              else
+                ftp_log (gftp_logging_error, NULL,
+                         _("Error: Child %d did not terminate properly\n"),
+                         pid);
 
 	      if (!ve_proc->view && !ve_proc->dontupload)
 		{
 		  /* We was editing the file. Upload it */
-		  if (stat (ve_proc->filename, &st) == -1)
+		  ret = stat (ve_proc->filename, &st);
+
+                  if (ret == -1)
 		    ftp_log (gftp_logging_error, NULL,
 		         _("Error: Cannot get information about file %s: %s\n"),
 			 ve_proc->filename, g_strerror (errno));
@@ -411,7 +457,7 @@ on_next_transfer (gftp_transfer * tdata)
   if (refresh_files && tdata->curfle && tdata->curfle->next &&
       compare_request (tdata->toreq, 
                        ((gftp_window_data *) tdata->towdata)->request, 1))
-    gftpui_refresh (tdata->towdata);
+    gftpui_refresh (tdata->towdata, 1);
 }
 
 
@@ -501,8 +547,7 @@ show_transfer (gftp_transfer * tdata)
                                    transdata);
     }
 
-  if (!tdata->toreq->stopable && tdata->toreq->need_userpass &&
-      (tdata->toreq->password == NULL || *tdata->toreq->password == '\0'))
+  if (!tdata->toreq->stopable && gftp_need_password (tdata->toreq))
     {
       tdata->toreq->stopable = 1;
       MakeEditDialog (_("Enter Password"),
@@ -512,8 +557,7 @@ show_transfer (gftp_transfer * tdata)
 		      cancel_get_trans_password, tdata);
     }
 
-  if (!tdata->fromreq->stopable && tdata->fromreq->need_userpass &&
-      (tdata->fromreq->password == NULL || *tdata->fromreq->password == '\0'))
+  if (!tdata->fromreq->stopable && gftp_need_password (tdata->fromreq))
     {
       tdata->fromreq->stopable = 1;
       MakeEditDialog (_("Enter Password"),
@@ -529,7 +573,6 @@ static void
 transfer_done (GList * node)
 {
   gftpui_common_curtrans_data * transdata;
-  gftp_request * fromreq;
   gftp_transfer * tdata;
   gftp_file * tempfle;
   GList * templist;
@@ -537,8 +580,6 @@ transfer_done (GList * node)
   tdata = node->data;
   if (tdata->started)
     {
-      fromreq = tdata->fromwdata != NULL ? ((gftp_window_data *) tdata->fromwdata)->request : NULL;
-
       if (GFTP_IS_SAME_HOST_STOP_TRANS ((gftp_window_data *) tdata->fromwdata,
                                          tdata->fromreq))
         {
@@ -563,7 +604,7 @@ transfer_done (GList * node)
 
       if (tdata->towdata != NULL && compare_request (tdata->toreq,
                            ((gftp_window_data *) tdata->towdata)->request, 1))
-        gftpui_refresh (tdata->towdata);
+        gftpui_refresh (tdata->towdata, 1);
 
       num_transfers_in_progress--;
     }
@@ -645,9 +686,10 @@ create_transfer (gftp_transfer * tdata)
 static void
 update_file_status (gftp_transfer * tdata)
 {
-  char totstr[100], dlstr[100], winstr[150], gotstr[50], ofstr[50];
-  int hours, mins, secs, pcent, st, show_trans_in_title;
+  char totstr[150], dlstr[150], winstr[150], gotstr[50], ofstr[50];
   unsigned long remaining_secs, lkbs;
+  int hours, mins, secs, pcent, st;
+  intptr_t show_trans_in_title;
   gftp_file * tempfle;
   struct timeval tv;
   
@@ -679,7 +721,12 @@ update_file_status (gftp_transfer * tdata)
   else
     pcent = 0;
 
-  g_snprintf (totstr, sizeof (totstr),
+  if (pcent > 100)
+    g_snprintf (totstr, sizeof (totstr),
+	_("Unknown percentage complete. (File %ld of %ld)"),
+	tdata->current_file_number, tdata->numdirs + tdata->numfiles);
+  else
+    g_snprintf (totstr, sizeof (totstr),
 	_("%d%% complete, %02d:%02d:%02d est. time remaining. (File %ld of %ld)"),
 	pcent, hours, mins, secs, tdata->current_file_number,
 	tdata->numdirs + tdata->numfiles);
@@ -778,7 +825,7 @@ update_downloads (gpointer data)
   if (window2.request->gotbytes != 0)
     update_window_transfer_bytes (&window2);
 
-  if (viewedit_process_done)
+  if (gftpui_common_child_process_done)
     check_done_process ();
 
   for (templist = gftp_file_transfers; templist != NULL;)
@@ -832,7 +879,7 @@ start_transfer (gpointer data)
 
   if (GTK_CLIST (dlwdw)->selection == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	       _("There are no file transfers selected\n"));
       return;
     }
@@ -854,7 +901,7 @@ stop_transfer (gpointer data)
 
   if (GTK_CLIST (dlwdw)->selection == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	      _("There are no file transfers selected\n"));
       return;
     }
@@ -888,7 +935,7 @@ skip_transfer (gpointer data)
 
   if (GTK_CLIST (dlwdw)->selection == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	      _("There are no file transfers selected\n"));
       return;
     }
@@ -928,7 +975,7 @@ remove_file_transfer (gpointer data)
 
   if (GTK_CLIST (dlwdw)->selection == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
               _("There are no file transfers selected\n"));
       return;
     }
@@ -981,7 +1028,7 @@ move_transfer_up (gpointer data)
 
   if (GTK_CLIST (dlwdw)->selection == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	      _("There are no file transfers selected\n"));
       return;
     }
@@ -1041,7 +1088,7 @@ move_transfer_down (gpointer data)
 
   if (GTK_CLIST (dlwdw)->selection == NULL)
     {
-      ftp_log (gftp_logging_misc, NULL,
+      ftp_log (gftp_logging_error, NULL,
 	      _("There are no file transfers selected\n"));
       return;
     }
